'''
Created on Jul 9, 2014

@author: roj-idl71
'''
import os
import datetime
import numpy

from schainpy.model.graphics.jroplot_base import Plot


class SpectraHeisScope(Plot):


    isConfig = None
    __nsubplots = None

    WIDTHPROF = None
    HEIGHTPROF = None
    PREFIX = 'spc'

    def __init__(self):#, **kwargs):

        Plot.__init__(self)#, **kwargs)
        self.isConfig = False
        self.__nsubplots = 1

        self.WIDTH = 230
        self.HEIGHT = 250
        self.WIDTHPROF = 120
        self.HEIGHTPROF = 0
        self.counter_imagwr = 0

        self.PLOT_CODE = SPEC_CODE

    def getSubplots(self):

        ncol = int(numpy.sqrt(self.nplots)+0.9)
        nrow = int(self.nplots*1./ncol + 0.9)

        return nrow, ncol

    def setup(self, id, nplots, wintitle, show):

        showprofile = False
        self.__showprofile = showprofile
        self.nplots = nplots

        ncolspan = 1
        colspan = 1
        if showprofile:
            ncolspan = 3
            colspan = 2
            self.__nsubplots = 2

        self.createFigure(id = id,
                          wintitle = wintitle,
                          widthplot = self.WIDTH + self.WIDTHPROF,
                          heightplot = self.HEIGHT + self.HEIGHTPROF,
                          show = show)

        nrow, ncol = self.getSubplots()

        counter = 0
        for y in range(nrow):
            for x in range(ncol):

                if counter >= self.nplots:
                    break

                self.addAxes(nrow, ncol*ncolspan, y, x*ncolspan, colspan, 1)

                if showprofile:
                    self.addAxes(nrow, ncol*ncolspan, y, x*ncolspan+colspan, 1, 1)

                counter += 1


    def run(self, dataOut, id, wintitle="", channelList=None,
            xmin=None, xmax=None, ymin=None, ymax=None, save=False,
            figpath='./', figfile=None, ftp=False, wr_period=1, show=True,
            server=None, folder=None, username=None, password=None,
            ftp_wei=0, exp_code=0, sub_exp_code=0, plot_pos=0):

        """

        Input:
            dataOut         :
            id        :
            wintitle        :
            channelList     :
            xmin            :    None,
            xmax            :    None,
            ymin            :    None,
            ymax            :    None,
        """

        if dataOut.flagNoData:
            return dataOut

        if dataOut.realtime:
            if not(isRealtime(utcdatatime = dataOut.utctime)):
                print('Skipping this plot function')
                return

        if channelList == None:
            channelIndexList = dataOut.channelIndexList
        else:
            channelIndexList = []
            for channel in channelList:
                if channel not in dataOut.channelList:
                    raise ValueError("Channel %d is not in dataOut.channelList")
                channelIndexList.append(dataOut.channelList.index(channel))

#        x = dataOut.heightList
        c   =  3E8
        deltaHeight = dataOut.heightList[1] - dataOut.heightList[0]
        #deberia cambiar para el caso de 1Mhz y 100KHz
        x = numpy.arange(-1*dataOut.nHeights/2.,dataOut.nHeights/2.)*(c/(2*deltaHeight*dataOut.nHeights*1000))
        #para 1Mhz descomentar la siguiente linea
        #x= x/(10000.0)
#        y = dataOut.data[channelIndexList,:] * numpy.conjugate(dataOut.data[channelIndexList,:])
#        y = y.real
        factor = dataOut.normFactor
        data = dataOut.data_spc / factor
        datadB = 10.*numpy.log10(data)
        y = datadB

        #thisDatetime = dataOut.datatime
        thisDatetime = datetime.datetime.utcfromtimestamp(dataOut.getTimeRange()[0])
        title = wintitle + " Scope: %s" %(thisDatetime.strftime("%d-%b-%Y %H:%M:%S"))
        xlabel = ""
        #para 1Mhz descomentar la siguiente linea
        #xlabel = "Frequency x 10000"
        ylabel = "Intensity (dB)"

        if not self.isConfig:
            nplots = len(channelIndexList)

            self.setup(id=id,
                       nplots=nplots,
                       wintitle=wintitle,
                       show=show)

            if xmin == None: xmin = numpy.nanmin(x)
            if xmax == None: xmax = numpy.nanmax(x)
            if ymin == None: ymin = numpy.nanmin(y)
            if ymax == None: ymax = numpy.nanmax(y)

            self.FTP_WEI = ftp_wei
            self.EXP_CODE = exp_code
            self.SUB_EXP_CODE = sub_exp_code
            self.PLOT_POS = plot_pos

            self.isConfig = True

        self.setWinTitle(title)

        for i in range(len(self.axesList)):
            ychannel = y[i,:]
            str_datetime = '%s %s'%(thisDatetime.strftime("%Y/%m/%d"),thisDatetime.strftime("%H:%M:%S"))
            title = "Channel %d: %4.2fdB: %s" %(dataOut.channelList[channelIndexList[i]], numpy.max(ychannel), str_datetime)
            axes = self.axesList[i]
            axes.pline(x, ychannel,
                        xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax,
                        xlabel=xlabel, ylabel=ylabel, title=title, grid='both')


        self.draw()

        self.save(figpath=figpath,
                  figfile=figfile,
                  save=save,
                  ftp=ftp,
                  wr_period=wr_period,
                  thisDatetime=thisDatetime)

        return dataOut


class RTIfromSpectraHeis(Plot):

    isConfig = None
    __nsubplots = None

    PREFIX = 'rtinoise'

    def __init__(self):#, **kwargs):
        Plot.__init__(self)#, **kwargs)
        self.timerange = 24*60*60
        self.isConfig = False
        self.__nsubplots = 1

        self.WIDTH = 820
        self.HEIGHT = 200
        self.WIDTHPROF = 120
        self.HEIGHTPROF = 0
        self.counter_imagwr = 0
        self.xdata = None
        self.ydata = None
        self.figfile = None

        self.PLOT_CODE = RTI_CODE

    def getSubplots(self):

        ncol = 1
        nrow = 1

        return nrow, ncol

    def setup(self, id, nplots, wintitle, showprofile=True, show=True):

        self.__showprofile = showprofile
        self.nplots = nplots

        ncolspan = 7
        colspan = 6
        self.__nsubplots = 2

        self.createFigure(id = id,
                          wintitle = wintitle,
                          widthplot = self.WIDTH+self.WIDTHPROF,
                          heightplot = self.HEIGHT+self.HEIGHTPROF,
                          show = show)

        nrow, ncol = self.getSubplots()

        self.addAxes(nrow, ncol*ncolspan, 0, 0, colspan, 1)


    def run(self, dataOut, id, wintitle="", channelList=None, showprofile='True',
            xmin=None, xmax=None, ymin=None, ymax=None,
            timerange=None,
            save=False, figpath='./', figfile=None, ftp=False, wr_period=1, show=True,
            server=None, folder=None, username=None, password=None,
            ftp_wei=0, exp_code=0, sub_exp_code=0, plot_pos=0):

        if dataOut.flagNoData:
            return dataOut


        if channelList == None:
            channelIndexList = dataOut.channelIndexList
            channelList = dataOut.channelList
        else:
            channelIndexList = []
            for channel in channelList:
                if channel not in dataOut.channelList:
                    raise ValueError("Channel %d is not in dataOut.channelList")
                channelIndexList.append(dataOut.channelList.index(channel))

        if timerange != None:
            self.timerange = timerange

        x = dataOut.getTimeRange()
        y = dataOut.getHeiRange()

        factor = dataOut.normFactor
        data = dataOut.data_spc / factor
        data = numpy.average(data,axis=1)
        datadB = 10*numpy.log10(data)

#        factor = dataOut.normFactor
#        noise = dataOut.getNoise()/factor
#        noisedB = 10*numpy.log10(noise)

        #thisDatetime = dataOut.datatime
        thisDatetime = datetime.datetime.utcfromtimestamp(dataOut.getTimeRange()[0])
        title = wintitle + " RTI: %s" %(thisDatetime.strftime("%d-%b-%Y"))
        xlabel = "Local Time"
        ylabel = "Intensity (dB)"

        if not self.isConfig:

            nplots = 1

            self.setup(id=id,
                       nplots=nplots,
                       wintitle=wintitle,
                       showprofile=showprofile,
                       show=show)

            self.tmin, self.tmax = self.getTimeLim(x, xmin, xmax)

            if ymin == None: ymin = numpy.nanmin(datadB)
            if ymax == None: ymax = numpy.nanmax(datadB)

            self.name = thisDatetime.strftime("%Y%m%d_%H%M%S")
            self.isConfig = True
            self.figfile = figfile
            self.xdata = numpy.array([])
            self.ydata = numpy.array([])

            self.FTP_WEI = ftp_wei
            self.EXP_CODE = exp_code
            self.SUB_EXP_CODE = sub_exp_code
            self.PLOT_POS = plot_pos

        self.setWinTitle(title)


#        title = "RTI %s" %(thisDatetime.strftime("%d-%b-%Y"))
        title = "RTI - %s" %(thisDatetime.strftime("%d-%b-%Y %H:%M:%S"))

        legendlabels = ["channel %d"%idchannel for idchannel in channelList]
        axes = self.axesList[0]

        self.xdata = numpy.hstack((self.xdata, x[0:1]))

        if len(self.ydata)==0:
            self.ydata = datadB[channelIndexList].reshape(-1,1)
        else:
            self.ydata = numpy.hstack((self.ydata, datadB[channelIndexList].reshape(-1,1)))


        axes.pmultilineyaxis(x=self.xdata, y=self.ydata,
                    xmin=self.tmin, xmax=self.tmax, ymin=ymin, ymax=ymax,
                    xlabel=xlabel, ylabel=ylabel, title=title, legendlabels=legendlabels, marker='.', markersize=8, linestyle="solid", grid='both',
                    XAxisAsTime=True
                    )

        self.draw()

        update_figfile = False

        if dataOut.ltctime >= self.tmax:
            self.counter_imagwr = wr_period
            self.isConfig = False
            update_figfile = True

        self.save(figpath=figpath,
                  figfile=figfile,
                  save=save,
                  ftp=ftp,
                  wr_period=wr_period,
                  thisDatetime=thisDatetime,
                  update_figfile=update_figfile)


        return dataOut


'''
Created on Jul 9, 2014
Modified on May 10, 2020

@author: Juan C. Espinoza
'''

import os
import datetime
import numpy

from schainpy.model.graphics.jroplot_base import Plot, plt


class SpectraPlot(Plot):
    '''
    Plot for Spectra data
    '''

    CODE = 'spc'
    colormap = 'jet'
    plot_type = 'pcolor'

    def setup(self):
        self.nplots = len(self.data.channels)
        self.ncols = int(numpy.sqrt(self.nplots) + 0.9)
        self.nrows = int((1.0 * self.nplots / self.ncols) + 0.9)
        self.height = 2.6 * self.nrows
        self.cb_label = 'dB'
        if self.showprofile:
            self.width = 4 * self.ncols
        else:
            self.width = 3.5 * self.ncols
        self.plots_adjust.update({'wspace': 0.4, 'hspace':0.4, 'left': 0.1, 'right': 0.9, 'bottom': 0.08})
        self.ylabel = 'Range [km]'

    def plot(self):
        if self.xaxis == "frequency":
            x = self.data.xrange[0]
            self.xlabel = "Frequency (kHz)"
        elif self.xaxis == "time":
            x = self.data.xrange[1]
            self.xlabel = "Time (ms)"
        else:
            x = self.data.xrange[2]
            self.xlabel = "Velocity (m/s)"

        if self.CODE == 'spc_moments':
            x = self.data.xrange[2]
            self.xlabel = "Velocity (m/s)"

        self.titles = []

        y = self.data.heights
        self.y = y
        z = self.data['spc']

        for n, ax in enumerate(self.axes):
            noise = self.data['noise'][n][-1]
            if self.CODE == 'spc_moments':
                mean = self.data['moments'][n, :, 1, :][-1]
            if ax.firsttime:
                self.xmax = self.xmax if self.xmax else numpy.nanmax(x)
                self.xmin = self.xmin if self.xmin else -self.xmax
                self.zmin = self.zmin if self.zmin else numpy.nanmin(z)
                self.zmax = self.zmax if self.zmax else numpy.nanmax(z)
                ax.plt = ax.pcolormesh(x, y, z[n].T,
                                       vmin=self.zmin,
                                       vmax=self.zmax,
                                       cmap=plt.get_cmap(self.colormap)
                                       )

                if self.showprofile:
                    ax.plt_profile = self.pf_axes[n].plot(
                        self.data['rti'][n][-1], y)[0]
                    ax.plt_noise = self.pf_axes[n].plot(numpy.repeat(noise, len(y)), y,
                                                        color="k", linestyle="dashed", lw=1)[0]
                if self.CODE == 'spc_moments':
                    ax.plt_mean = ax.plot(mean, y, color='k')[0]
            else:
                ax.plt.set_array(z[n].T.ravel())
                if self.showprofile:
                    ax.plt_profile.set_data(self.data['rti'][n][-1], y)
                    ax.plt_noise.set_data(numpy.repeat(noise, len(y)), y)
                if self.CODE == 'spc_moments':
                    ax.plt_mean.set_data(mean, y)
            self.titles.append('CH {}: {:3.2f}dB'.format(n, noise))


class CrossSpectraPlot(Plot):

    CODE = 'cspc'
    colormap = 'jet'
    plot_type = 'pcolor'
    zmin_coh = None
    zmax_coh = None
    zmin_phase = None
    zmax_phase = None

    def setup(self):

        self.ncols = 4
        self.nrows = len(self.data.pairs)
        self.nplots = self.nrows * 4
        self.width = 3.1 * self.ncols
        self.height = 2.6 * self.nrows
        self.ylabel = 'Range [km]'
        self.showprofile = False
        self.plots_adjust.update({'left': 0.08, 'right': 0.92, 'wspace': 0.5, 'hspace':0.4, 'top':0.95, 'bottom': 0.08})

    def plot(self):

        if self.xaxis == "frequency":
            x = self.data.xrange[0]
            self.xlabel = "Frequency (kHz)"
        elif self.xaxis == "time":
            x = self.data.xrange[1]
            self.xlabel = "Time (ms)"
        else:
            x = self.data.xrange[2]
            self.xlabel = "Velocity (m/s)"
        
        self.titles = []

        y = self.data.heights
        self.y = y
        nspc = self.data['spc']
        spc = self.data['cspc'][0]
        cspc = self.data['cspc'][1]

        for n in range(self.nrows):
            noise = self.data['noise'][:,-1]
            pair = self.data.pairs[n]
            ax = self.axes[4 * n]
            if ax.firsttime:                                               
                self.xmax = self.xmax if self.xmax else numpy.nanmax(x)    
                self.xmin = self.xmin if self.xmin else -self.xmax         
                self.zmin = self.zmin if self.zmin else numpy.nanmin(nspc)  
                self.zmax = self.zmax if self.zmax else numpy.nanmax(nspc)  
                ax.plt = ax.pcolormesh(x , y , nspc[pair[0]].T,
                                       vmin=self.zmin,
                                       vmax=self.zmax,
                                       cmap=plt.get_cmap(self.colormap)
                                       )                                   
            else:                                                          
                ax.plt.set_array(nspc[pair[0]].T.ravel())
            self.titles.append('CH {}: {:3.2f}dB'.format(pair[0], noise[pair[0]]))

            ax = self.axes[4 * n + 1]
            if ax.firsttime:
                ax.plt = ax.pcolormesh(x , y, nspc[pair[1]].T,
                                       vmin=self.zmin,
                                       vmax=self.zmax,
                                       cmap=plt.get_cmap(self.colormap)
                                       )
            else: 
                ax.plt.set_array(nspc[pair[1]].T.ravel())
            self.titles.append('CH {}: {:3.2f}dB'.format(pair[1], noise[pair[1]]))
            
            out = cspc[n] / numpy.sqrt(spc[pair[0]] * spc[pair[1]])
            coh = numpy.abs(out)
            phase = numpy.arctan2(out.imag, out.real) * 180 / numpy.pi

            ax = self.axes[4 * n + 2]
            if ax.firsttime:
                ax.plt = ax.pcolormesh(x, y, coh.T,
                                       vmin=0,
                                       vmax=1,
                                       cmap=plt.get_cmap(self.colormap_coh)
                                       )
            else:
                ax.plt.set_array(coh.T.ravel())
            self.titles.append(
                'Coherence Ch{} * Ch{}'.format(pair[0], pair[1]))
                                   
            ax = self.axes[4 * n + 3]
            if ax.firsttime:
                ax.plt = ax.pcolormesh(x, y, phase.T,
                                       vmin=-180,
                                       vmax=180,
                                       cmap=plt.get_cmap(self.colormap_phase) 
                                       )
            else:
                ax.plt.set_array(phase.T.ravel())
            self.titles.append('Phase CH{} * CH{}'.format(pair[0], pair[1]))


class RTIPlot(Plot):
    '''
    Plot for RTI data
    '''

    CODE = 'rti'
    colormap = 'jet'
    plot_type = 'pcolorbuffer'

    def setup(self):
        self.xaxis = 'time'
        self.ncols = 1
        self.nrows = len(self.data.channels)
        self.nplots = len(self.data.channels)
        self.ylabel = 'Range [km]'
        self.xlabel = 'Time'
        self.cb_label = 'dB'
        self.plots_adjust.update({'hspace':0.8, 'left': 0.1, 'bottom': 0.08, 'right':0.95})
        self.titles = ['{} Channel {}'.format(
            self.CODE.upper(), x) for x in range(self.nrows)]

    def plot(self):
        self.x = self.data.times
        self.y = self.data.heights
        self.z = self.data[self.CODE]
        self.z = numpy.ma.masked_invalid(self.z)

        if self.decimation is None:
            x, y, z = self.fill_gaps(self.x, self.y, self.z)
        else:
            x, y, z = self.fill_gaps(*self.decimate())

        for n, ax in enumerate(self.axes):
            self.zmin = self.zmin if self.zmin else numpy.min(self.z)
            self.zmax = self.zmax if self.zmax else numpy.max(self.z)
            if ax.firsttime:
                ax.plt = ax.pcolormesh(x, y, z[n].T,
                                       vmin=self.zmin,
                                       vmax=self.zmax,
                                       cmap=plt.get_cmap(self.colormap)
                                       )
                if self.showprofile:
                    ax.plot_profile = self.pf_axes[n].plot(
                        self.data['rti'][n][-1], self.y)[0]
                    ax.plot_noise = self.pf_axes[n].plot(numpy.repeat(self.data['noise'][n][-1], len(self.y)), self.y,
                                                         color="k", linestyle="dashed", lw=1)[0]
            else:
                ax.collections.remove(ax.collections[0])
                ax.plt = ax.pcolormesh(x, y, z[n].T,
                                       vmin=self.zmin,
                                       vmax=self.zmax,
                                       cmap=plt.get_cmap(self.colormap)
                                       )
                if self.showprofile:
                    ax.plot_profile.set_data(self.data['rti'][n][-1], self.y)
                    ax.plot_noise.set_data(numpy.repeat(
                        self.data['noise'][n][-1], len(self.y)), self.y)


class CoherencePlot(RTIPlot):
    '''
    Plot for Coherence data
    '''

    CODE = 'coh'

    def setup(self):
        self.xaxis = 'time'
        self.ncols = 1
        self.nrows = len(self.data.pairs)
        self.nplots = len(self.data.pairs)
        self.ylabel = 'Range [km]'
        self.xlabel = 'Time'
        self.plots_adjust.update({'hspace':0.6, 'left': 0.1, 'bottom': 0.1,'right':0.95})
        if self.CODE == 'coh':
            self.cb_label = ''
            self.titles = [
                'Coherence Map Ch{} * Ch{}'.format(x[0], x[1]) for x in self.data.pairs]
        else:
            self.cb_label = 'Degrees'
            self.titles = [
                'Phase Map Ch{} * Ch{}'.format(x[0], x[1]) for x in self.data.pairs]


class PhasePlot(CoherencePlot):
    '''
    Plot for Phase map data
    '''

    CODE = 'phase'
    colormap = 'seismic'


class NoisePlot(Plot):
    '''
    Plot for noise 
    '''

    CODE = 'noise'
    plot_type = 'scatterbuffer'


    def setup(self):
        self.xaxis = 'time'
        self.ncols = 1
        self.nrows = 1
        self.nplots = 1
        self.ylabel = 'Intensity [dB]'
        self.xlabel = 'Time'
        self.titles = ['Noise']
        self.colorbar = False

    def plot(self):

        x = self.data.times
        xmin = self.data.min_time
        xmax = xmin + self.xrange * 60 * 60
        Y = self.data[self.CODE]

        if self.axes[0].firsttime:
            for ch in self.data.channels:
                y = Y[ch]
                self.axes[0].plot(x, y, lw=1, label='Ch{}'.format(ch))
            plt.legend()
        else:
            for ch in self.data.channels:
                y = Y[ch]
                self.axes[0].lines[ch].set_data(x, y)

        self.ymin = numpy.nanmin(Y) - 5
        self.ymax = numpy.nanmax(Y) + 5


class PowerProfilePlot(Plot):

    CODE = 'spcprofile'
    plot_type = 'scatter'
    buffering = False

    def setup(self):

        self.ncols = 1
        self.nrows = 1
        self.nplots = 1
        self.height = 4
        self.width = 3
        self.ylabel = 'Range [km]'
        self.xlabel = 'Intensity [dB]'
        self.titles = ['Power Profile']
        self.colorbar = False

    def plot(self):

        y = self.data.heights
        self.y = y

        x = self.data['spcprofile']
        
        if self.xmin is None: self.xmin = numpy.nanmin(x)*0.9
        if self.xmax is None: self.xmax = numpy.nanmax(x)*1.1
        
        if self.axes[0].firsttime:
            for ch in self.data.channels:
                self.axes[0].plot(x[ch], y, lw=1, label='Ch{}'.format(ch))
            plt.legend()
        else:
            for ch in self.data.channels:
                self.axes[0].lines[ch].set_data(x[ch], y)


class SpectraCutPlot(Plot):

    CODE = 'spc_cut'
    plot_type = 'scatter'
    buffering = False

    def setup(self):

        self.nplots = len(self.data.channels)
        self.ncols = int(numpy.sqrt(self.nplots) + 0.9)
        self.nrows = int((1.0 * self.nplots / self.ncols) + 0.9)
        self.width = 3.4 * self.ncols + 1.5
        self.height = 3 * self.nrows
        self.ylabel = 'Power [dB]'
        self.colorbar = False
        self.plots_adjust.update({'left':0.1, 'hspace':0.3, 'right': 0.75, 'bottom':0.08})

    def plot(self):
        if self.xaxis == "frequency":
            x = self.data.xrange[0][1:]
            self.xlabel = "Frequency (kHz)"
        elif self.xaxis == "time":
            x = self.data.xrange[1]
            self.xlabel = "Time (ms)"
        else:
            x = self.data.xrange[2]
            self.xlabel = "Velocity (m/s)"

        self.titles = []

        y = self.data.heights
        #self.y = y
        z = self.data['spc_cut']

        if self.height_index:
            index = numpy.array(self.height_index)
        else:
            index = numpy.arange(0, len(y), int((len(y))/9))

        for n, ax in enumerate(self.axes):
            if ax.firsttime:
                self.xmax = self.xmax if self.xmax else numpy.nanmax(x)
                self.xmin = self.xmin if self.xmin else -self.xmax
                self.ymin = self.ymin if self.ymin else numpy.nanmin(z)
                self.ymax = self.ymax if self.ymax else numpy.nanmax(z)
                ax.plt = ax.plot(x, z[n, :, index].T)
                labels = ['Range = {:2.1f}km'.format(y[i]) for i in index]
                self.figures[0].legend(ax.plt, labels, loc='center right')
            else:
                for i, line in enumerate(ax.plt):
                    line.set_data(x, z[n, :, i])
            self.titles.append('CH {}'.format(n))


class BeaconPhase(Plot):

    __isConfig = None
    __nsubplots = None

    PREFIX = 'beacon_phase'

    def __init__(self):
        Plot.__init__(self)
        self.timerange = 24*60*60
        self.isConfig = False
        self.__nsubplots = 1
        self.counter_imagwr = 0
        self.WIDTH = 800
        self.HEIGHT = 400
        self.WIDTHPROF = 120
        self.HEIGHTPROF = 0
        self.xdata = None
        self.ydata = None

        self.PLOT_CODE = BEACON_CODE

        self.FTP_WEI = None
        self.EXP_CODE = None
        self.SUB_EXP_CODE = None
        self.PLOT_POS = None

        self.filename_phase = None

        self.figfile = None

        self.xmin = None
        self.xmax = None

    def getSubplots(self):

        ncol = 1
        nrow = 1

        return nrow, ncol

    def setup(self, id, nplots, wintitle, showprofile=True, show=True):

        self.__showprofile = showprofile
        self.nplots = nplots

        ncolspan = 7
        colspan = 6
        self.__nsubplots = 2

        self.createFigure(id = id,
                          wintitle = wintitle,
                          widthplot = self.WIDTH+self.WIDTHPROF,
                          heightplot = self.HEIGHT+self.HEIGHTPROF,
                          show=show)

        nrow, ncol = self.getSubplots()

        self.addAxes(nrow, ncol*ncolspan, 0, 0, colspan, 1)

    def save_phase(self, filename_phase):
        f = open(filename_phase,'w+')
        f.write('\n\n')
        f.write('JICAMARCA RADIO OBSERVATORY - Beacon Phase \n')
        f.write('DD MM YYYY  HH MM SS   pair(2,0) pair(2,1) pair(2,3) pair(2,4)\n\n' )
        f.close()

    def save_data(self, filename_phase, data, data_datetime):
        f=open(filename_phase,'a')
        timetuple_data = data_datetime.timetuple()
        day = str(timetuple_data.tm_mday)
        month = str(timetuple_data.tm_mon)
        year = str(timetuple_data.tm_year)
        hour = str(timetuple_data.tm_hour)
        minute = str(timetuple_data.tm_min)
        second = str(timetuple_data.tm_sec)
        f.write(day+' '+month+' '+year+'  '+hour+' '+minute+' '+second+'   '+str(data[0])+'   '+str(data[1])+'   '+str(data[2])+'   '+str(data[3])+'\n')
        f.close()

    def plot(self):
        log.warning('TODO: Not yet implemented...')

    def run(self, dataOut, id, wintitle="", pairsList=None, showprofile='True',
            xmin=None, xmax=None, ymin=None, ymax=None, hmin=None, hmax=None,
            timerange=None,
            save=False, figpath='./', figfile=None, show=True, ftp=False, wr_period=1,
            server=None, folder=None, username=None, password=None,
            ftp_wei=0, exp_code=0, sub_exp_code=0, plot_pos=0):

        if dataOut.flagNoData:         
            return dataOut

        if not isTimeInHourRange(dataOut.datatime, xmin, xmax):
            return

        if pairsList == None:
            pairsIndexList = dataOut.pairsIndexList[:10]
        else:
            pairsIndexList = []
            for pair in pairsList:
                if pair not in dataOut.pairsList:
                    raise ValueError("Pair %s is not in dataOut.pairsList" %(pair))
                pairsIndexList.append(dataOut.pairsList.index(pair))

        if pairsIndexList == []:
            return

 #         if len(pairsIndexList) > 4:
 #             pairsIndexList = pairsIndexList[0:4]

        hmin_index = None
        hmax_index = None

        if hmin != None and hmax != None:
            indexes = numpy.arange(dataOut.nHeights)
            hmin_list = indexes[dataOut.heightList >= hmin]
            hmax_list = indexes[dataOut.heightList <= hmax]

            if hmin_list.any():
                hmin_index = hmin_list[0]

            if hmax_list.any():
                hmax_index = hmax_list[-1]+1

        x = dataOut.getTimeRange()
        #y = dataOut.getHeiRange()


        thisDatetime = dataOut.datatime

        title = wintitle + " Signal Phase" # : %s" %(thisDatetime.strftime("%d-%b-%Y"))
        xlabel = "Local Time"
        ylabel = "Phase (degrees)"

        update_figfile = False

        nplots = len(pairsIndexList)
        #phase = numpy.zeros((len(pairsIndexList),len(dataOut.beacon_heiIndexList)))
        phase_beacon = numpy.zeros(len(pairsIndexList))
        for i in range(nplots):
            pair = dataOut.pairsList[pairsIndexList[i]]
            ccf = numpy.average(dataOut.data_cspc[pairsIndexList[i], :, hmin_index:hmax_index], axis=0)
            powa = numpy.average(dataOut.data_spc[pair[0], :, hmin_index:hmax_index], axis=0)
            powb = numpy.average(dataOut.data_spc[pair[1], :, hmin_index:hmax_index], axis=0)
            avgcoherenceComplex = ccf/numpy.sqrt(powa*powb)
            phase = numpy.arctan2(avgcoherenceComplex.imag, avgcoherenceComplex.real)*180/numpy.pi

            if dataOut.beacon_heiIndexList:
                phase_beacon[i] = numpy.average(phase[dataOut.beacon_heiIndexList])
            else:
                phase_beacon[i] = numpy.average(phase)

        if not self.isConfig:

            nplots = len(pairsIndexList)

            self.setup(id=id,
                       nplots=nplots,
                       wintitle=wintitle,
                       showprofile=showprofile,
                       show=show)

            if timerange != None:
                self.timerange = timerange

            self.xmin, self.xmax = self.getTimeLim(x, xmin, xmax, timerange)

            if ymin == None: ymin = 0
            if ymax == None: ymax = 360

            self.FTP_WEI = ftp_wei
            self.EXP_CODE = exp_code
            self.SUB_EXP_CODE = sub_exp_code
            self.PLOT_POS = plot_pos

            self.name = thisDatetime.strftime("%Y%m%d_%H%M%S")
            self.isConfig = True
            self.figfile = figfile
            self.xdata = numpy.array([])
            self.ydata = numpy.array([])

            update_figfile = True

            #open file beacon phase
            path = '%s%03d' %(self.PREFIX, self.id)
            beacon_file = os.path.join(path,'%s.txt'%self.name)
            self.filename_phase = os.path.join(figpath,beacon_file)
            #self.save_phase(self.filename_phase)


        #store data beacon phase
        #self.save_data(self.filename_phase, phase_beacon, thisDatetime)

        self.setWinTitle(title)


        title = "Phase Plot %s" %(thisDatetime.strftime("%Y/%m/%d %H:%M:%S"))

        legendlabels = ["Pair (%d,%d)"%(pair[0], pair[1]) for pair in dataOut.pairsList]

        axes = self.axesList[0]

        self.xdata = numpy.hstack((self.xdata, x[0:1]))

        if len(self.ydata)==0:
            self.ydata = phase_beacon.reshape(-1,1)
        else:
            self.ydata = numpy.hstack((self.ydata, phase_beacon.reshape(-1,1)))


        axes.pmultilineyaxis(x=self.xdata, y=self.ydata,
                    xmin=self.xmin, xmax=self.xmax, ymin=ymin, ymax=ymax,
                    xlabel=xlabel, ylabel=ylabel, title=title, legendlabels=legendlabels, marker='x', markersize=8, linestyle="solid",
                    XAxisAsTime=True, grid='both'
                    )

        self.draw()

        if dataOut.ltctime >= self.xmax:
            self.counter_imagwr = wr_period
            self.isConfig = False
            update_figfile = True

        self.save(figpath=figpath,
                  figfile=figfile,
                  save=save,
                  ftp=ftp,
                  wr_period=wr_period,
                  thisDatetime=thisDatetime,
                  update_figfile=update_figfile)

        return dataOut


import os
import sys
import glob
import fnmatch
import datetime
import time
import re
import h5py
import numpy

from scipy.optimize import curve_fit
from scipy import asarray as ar, exp
from scipy import stats

from duplicity.path import Path
from numpy.ma.core import getdata

SPEED_OF_LIGHT = 299792458
SPEED_OF_LIGHT = 3e8

try:
    from gevent import sleep
except:
    from time import sleep

from schainpy.model.data.jrodata import Spectra
#from schainpy.model.data.BLTRheaderIO import FileHeader, RecordHeader
from schainpy.model.proc.jroproc_base import ProcessingUnit, Operation
#from schainpy.model.io.jroIO_bltr import BLTRReader
from numpy import imag, shape, NaN


startFp = open(
    '/home/erick/Documents/MIRA35C/20160117/20160117_0000.zspc', "rb")


FILE_HEADER = numpy.dtype([  # HEADER 1024bytes
                          ('Hname', numpy.str_, 32),  # Original file name
                          # Date and time when the file was created
                          ('Htime', numpy.str_, 32),
                          # Name of operator who created the file
                          ('Hoper', numpy.str_, 64),
                          # Place where the measurements was carried out
                          ('Hplace', numpy.str_, 128),
                          # Description of measurements
                          ('Hdescr', numpy.str_, 256),
                          ('Hdummy', numpy.str_, 512),  # Reserved space
                          # Main chunk
                          ('Msign', '<i4'),  # Main chunk signature FZKF or NUIG
                          ('MsizeData', '<i4'),  # Size of data block main chunk
                          # Processing DSP parameters
                          ('PPARsign', '<i4'),  # PPAR signature
                          ('PPARsize', '<i4'),  # PPAR size of block
                          ('PPARprf', '<i4'),  # Pulse repetition frequency
                          ('PPARpdr', '<i4'),  # Pulse duration
                          ('PPARsft', '<i4'),  # FFT length
                          # Number of spectral (in-coherent) averages
                          ('PPARavc', '<i4'),
                          # Number of lowest range gate for moment estimation
                          ('PPARihp', '<i4'),
                          # Count for gates for moment estimation
                          ('PPARchg', '<i4'),
                          # switch on/off polarimetric measurements. Should be 1.
                          ('PPARpol', '<i4'),
                          # Service DSP parameters
                          # STC attenuation on the lowest ranges on/off
                          ('SPARatt', '<i4'),
                          ('SPARtx', '<i4'),  # OBSOLETE
                          ('SPARaddGain0', '<f4'),  # OBSOLETE
                          ('SPARaddGain1', '<f4'),  # OBSOLETE
                          # Debug only. It normal mode it is 0.
                          ('SPARwnd', '<i4'),
                          # Delay between sync pulse and tx pulse for phase corr, ns
                          ('SPARpos', '<i4'),
                          # "add to pulse" to compensate for delay between the leading edge of driver pulse and envelope of the RF signal.
                          ('SPARadd', '<i4'),
                          # Time for measuring txn pulse phase. OBSOLETE
                          ('SPARlen', '<i4'),
                          ('SPARcal', '<i4'),  # OBSOLETE
                          ('SPARnos', '<i4'),  # OBSOLETE
                          ('SPARof0', '<i4'),  # detection threshold
                          ('SPARof1', '<i4'),  # OBSOLETE
                          ('SPARswt', '<i4'),  # 2nd moment estimation threshold
                          ('SPARsum', '<i4'),  # OBSOLETE
                          ('SPARosc', '<i4'),  # flag Oscillosgram mode
                          ('SPARtst', '<i4'),  # OBSOLETE
                          ('SPARcor', '<i4'),  # OBSOLETE
                          ('SPARofs', '<i4'),  # OBSOLETE
                          # Hildebrand div noise detection on noise gate
                          ('SPARhsn', '<i4'),
                          # Hildebrand div noise detection on all gates
                          ('SPARhsa', '<f4'),
                          ('SPARcalibPow_M', '<f4'),  # OBSOLETE
                          ('SPARcalibSNR_M', '<f4'),  # OBSOLETE
                          ('SPARcalibPow_S', '<f4'),  # OBSOLETE
                          ('SPARcalibSNR_S', '<f4'),  # OBSOLETE
                          # Lowest range gate for spectra saving Raw_Gate1 >=5
                          ('SPARrawGate1', '<i4'),
                          # Number of range gates with atmospheric signal
                          ('SPARrawGate2', '<i4'),
                          # flag - IQ or spectra saving on/off
                          ('SPARraw', '<i4'),
                          ('SPARprc', '<i4'), ])  # flag - Moment estimation switched on/off


self.Hname = None
self.Htime = None
self.Hoper = None
self.Hplace = None
self.Hdescr = None
self.Hdummy = None

self.Msign = None
self.MsizeData = None

self.PPARsign = None
self.PPARsize = None
self.PPARprf = None
self.PPARpdr = None
self.PPARsft = None
self.PPARavc = None
self.PPARihp = None
self.PPARchg = None
self.PPARpol = None
# Service DSP parameters
self.SPARatt = None
self.SPARtx = None
self.SPARaddGain0 = None
self.SPARaddGain1 = None
self.SPARwnd = None
self.SPARpos = None
self.SPARadd = None
self.SPARlen = None
self.SPARcal = None
self.SPARnos = None
self.SPARof0 = None
self.SPARof1 = None
self.SPARswt = None
self.SPARsum = None
self.SPARosc = None
self.SPARtst = None
self.SPARcor = None
self.SPARofs = None
self.SPARhsn = None
self.SPARhsa = None
self.SPARcalibPow_M = None
self.SPARcalibSNR_M = None
self.SPARcalibPow_S = None
self.SPARcalibSNR_S = None
self.SPARrawGate1 = None
self.SPARrawGate2 = None
self.SPARraw = None
self.SPARprc = None


header = numpy.fromfile(fp, FILE_HEADER, 1)
'''      numpy.fromfile(file, dtype, count, sep='')
    file : file or str
    Open file object or filename.
    
    dtype : data-type
    Data type of the returned array. For binary files, it is used to determine 
    the size and byte-order of the items in the file.
    
    count : int
    Number of items to read. -1 means all items (i.e., the complete file).
    
    sep : str
    Separator between items if file is a text file. Empty ("") separator means 
    the file should be treated as binary. Spaces (" ") in the separator match zero 
    or more whitespace characters. A separator consisting only of spaces must match 
    at least one whitespace.
    
'''

Hname = str(header['Hname'][0])
Htime = str(header['Htime'][0])
Hoper = str(header['Hoper'][0])
Hplace = str(header['Hplace'][0])
Hdescr = str(header['Hdescr'][0])
Hdummy = str(header['Hdummy'][0])

Msign = header['Msign'][0]
MsizeData = header['MsizeData'][0]

PPARsign = header['PPARsign'][0]
PPARsize = header['PPARsize'][0]
PPARprf = header['PPARprf'][0]
PPARpdr = header['PPARpdr'][0]
PPARsft = header['PPARsft'][0]
PPARavc = header['PPARavc'][0]
PPARihp = header['PPARihp'][0]
PPARchg = header['PPARchg'][0]
PPARpol = header['PPARpol'][0]
# Service DSP parameters
SPARatt = header['SPARatt'][0]
SPARtx = header['SPARtx'][0]
SPARaddGain0 = header['SPARaddGain0'][0]
SPARaddGain1 = header['SPARaddGain1'][0]
SPARwnd = header['SPARwnd'][0]
SPARpos = header['SPARpos'][0]
SPARadd = header['SPARadd'][0]
SPARlen = header['SPARlen'][0]
SPARcal = header['SPARcal'][0]
SPARnos = header['SPARnos'][0]
SPARof0 = header['SPARof0'][0]
SPARof1 = header['SPARof1'][0]
SPARswt = header['SPARswt'][0]
SPARsum = header['SPARsum'][0]
SPARosc = header['SPARosc'][0]
SPARtst = header['SPARtst'][0]
SPARcor = header['SPARcor'][0]
SPARofs = header['SPARofs'][0]
SPARhsn = header['SPARhsn'][0]
SPARhsa = header['SPARhsa'][0]
SPARcalibPow_M = header['SPARcalibPow_M'][0]
SPARcalibSNR_M = header['SPARcalibSNR_M'][0]
SPARcalibPow_S = header['SPARcalibPow_S'][0]
SPARcalibSNR_S = header['SPARcalibSNR_S'][0]
SPARrawGate1 = header['SPARrawGate1'][0]
SPARrawGate2 = header['SPARrawGate2'][0]
SPARraw = header['SPARraw'][0]
SPARprc = header['SPARprc'][0]


SRVI_STRUCTURE = numpy.dtype([
                            ('frame_cnt', '<u4'),
                            ('time_t', '<u4'),   #
                            ('tpow', '<f4'),     #
                            ('npw1', '<f4'),     #
                            ('npw2', '<f4'),     #
                            ('cpw1', '<f4'),     #
                            ('pcw2', '<f4'),     #
                            ('ps_err', '<u4'),   #
                            ('te_err', '<u4'),   #
                            ('rc_err', '<u4'),   #
                            ('grs1', '<u4'),     #
                            ('grs2', '<u4'),     #
                            ('azipos', '<f4'),     #
                            ('azivel', '<f4'),     #
                            ('elvpos', '<f4'),     #
                            ('elvvel', '<f4'),     #
                            ('northAngle', '<f4'),
                            ('microsec', '<u4'),   #
                            ('azisetvel', '<f4'),  #
                            ('elvsetpos', '<f4'),  #
                            ('RadarConst', '<f4'), ])   #

JUMP_STRUCTURE = numpy.dtype([
                            ('jump', '<u140'),
                            ('SizeOfDataBlock1', numpy.str_, 32),
                            ('jump', '<i4'),
                            ('DataBlockTitleSRVI1', numpy.str_, 32),
                            ('SizeOfSRVI1', '<i4'), ])


# frame_cnt=0,  time_t= 0,  tpow=0,   npw1=0,   npw2=0,
# cpw1=0,   pcw2=0,       ps_err=0,   te_err=0,   rc_err=0,   grs1=0,
# grs2=0,   azipos=0,   azivel=0,   elvpos=0,   elvvel=0,   northangle=0,
# microsec=0,   azisetvel=0,   elvsetpos=0,   RadarConst=0


frame_cnt = frame_cnt
dwell = time_t
tpow = tpow
npw1 = npw1
npw2 = npw2
cpw1 = cpw1
pcw2 = pcw2
ps_err = ps_err
te_err = te_err
rc_err = rc_err
grs1 = grs1
grs2 = grs2
azipos = azipos
azivel = azivel
elvpos = elvpos
elvvel = elvvel
northAngle = northAngle
microsec = microsec
azisetvel = azisetvel
elvsetpos = elvsetpos
RadarConst5 = RadarConst


# print fp
# startFp = open('/home/erick/Documents/Data/huancayo.20161019.22.fdt',"rb") #The method tell() returns the current position of the file read/write pointer within the file.
# startFp = open(fp,"rb") #The method tell() returns the current position of the file read/write pointer within the file.
# RecCounter=0
# Off2StartNxtRec=811248
# print 'OffsetStartHeader ',self.OffsetStartHeader,'RecCounter ', self.RecCounter, 'Off2StartNxtRec ' , self.Off2StartNxtRec
#OffRHeader= self.OffsetStartHeader + self.RecCounter*self.Off2StartNxtRec
#startFp.seek(OffRHeader, os.SEEK_SET)
print('debe ser 48, RecCounter*811248', self.OffsetStartHeader, self.RecCounter, self.Off2StartNxtRec)
print('Posicion del bloque:        ', OffRHeader)

header = numpy.fromfile(startFp, SRVI_STRUCTURE, 1)

self.frame_cnt = header['frame_cnt'][0]
self.time_t = header['frame_cnt'][0]   #
self.tpow = header['frame_cnt'][0]     #
self.npw1 = header['frame_cnt'][0]     #
self.npw2 = header['frame_cnt'][0]     #
self.cpw1 = header['frame_cnt'][0]     #
self.pcw2 = header['frame_cnt'][0]     #
self.ps_err = header['frame_cnt'][0]    #
self.te_err = header['frame_cnt'][0]    #
self.rc_err = header['frame_cnt'][0]    #
self.grs1 = header['frame_cnt'][0]      #
self.grs2 = header['frame_cnt'][0]      #
self.azipos = header['frame_cnt'][0]     #
self.azivel = header['frame_cnt'][0]     #
self.elvpos = header['frame_cnt'][0]     #
self.elvvel = header['frame_cnt'][0]     #
self.northAngle = header['frame_cnt'][0]    #
self.microsec = header['frame_cnt'][0]      #
self.azisetvel = header['frame_cnt'][0]     #
self.elvsetpos = header['frame_cnt'][0]     #
self.RadarConst = header['frame_cnt'][0]    #


self.ipp = 0.5 * (SPEED_OF_LIGHT / self.PRFhz)

self.RHsize = 180 + 20 * self.nChannels
self.Datasize = self.nProfiles * self.nChannels * self.nHeights * 2 * 4
# print 'Datasize',self.Datasize
endFp = self.OffsetStartHeader + self.RecCounter * self.Off2StartNxtRec

print('==============================================')

print('==============================================')


'''
Created on Nov 9, 2016

@author: roj- LouVD
'''


import os
import sys
import time
import glob
import datetime

import numpy

import schainpy.admin
from schainpy.model.proc.jroproc_base import ProcessingUnit, MPDecorator
from schainpy.model.data.jrodata import Parameters
from schainpy.model.io.jroIO_base import Reader
from schainpy.utils import log

FILE_HEADER_STRUCTURE = numpy.dtype([
    ('FMN', '<u4'),
    ('nrec', '<u4'),
    ('fr_offset', '<u4'),
    ('id', '<u4'),
    ('site', 'u1', (32,))
])

REC_HEADER_STRUCTURE = numpy.dtype([
    ('rmn', '<u4'),
    ('rcounter', '<u4'),
    ('nr_offset', '<u4'),
    ('tr_offset', '<u4'),
    ('time', '<u4'),
    ('time_msec', '<u4'),
    ('tag', 'u1', (32,)),
    ('comments', 'u1', (32,)),
    ('lat', '<f4'),
    ('lon', '<f4'),
    ('gps_status', '<u4'),
    ('freq', '<u4'),
    ('freq0', '<u4'),
    ('nchan', '<u4'),
    ('delta_r', '<u4'),
    ('nranges', '<u4'),
    ('r0', '<u4'),
    ('prf', '<u4'),
    ('ncoh', '<u4'),
    ('npoints', '<u4'),
    ('polarization', '<i4'),
    ('rx_filter', '<u4'),
    ('nmodes', '<u4'),
    ('dmode_index', '<u4'),
    ('dmode_rngcorr', '<u4'),
    ('nrxs', '<u4'),
    ('acf_length', '<u4'),
    ('acf_lags', '<u4'),
    ('sea_to_atmos', '<f4'),
    ('sea_notch', '<u4'),
    ('lh_sea', '<u4'),
    ('hh_sea', '<u4'),
    ('nbins_sea', '<u4'),
    ('min_snr', '<f4'),
    ('min_cc', '<f4'),
    ('max_time_diff', '<f4')
])

DATA_STRUCTURE = numpy.dtype([
    ('range', '<u4'),
    ('status', '<u4'),
    ('zonal', '<f4'),
    ('meridional', '<f4'),
    ('vertical', '<f4'),
    ('zonal_a', '<f4'),
    ('meridional_a', '<f4'),
    ('corrected_fading', '<f4'),  # seconds
    ('uncorrected_fading', '<f4'),  # seconds
    ('time_diff', '<f4'),
    ('major_axis', '<f4'),
    ('axial_ratio', '<f4'),
    ('orientation', '<f4'),
    ('sea_power', '<u4'),
    ('sea_algorithm', '<u4')
])


class BLTRParamReader(Reader, ProcessingUnit):
    '''
    Boundary Layer and Tropospheric Radar (BLTR) reader, Wind velocities and SNR 
    from *.sswma files
    '''

    ext = '.sswma'

    def __init__(self):

        ProcessingUnit.__init__(self)

        self.dataOut = Parameters()
        self.dataOut.timezone = 300
        self.counter_records = 0
        self.flagNoMoreFiles = 0
        self.isConfig = False
        self.filename = None
        self.status_value = 0
        self.datatime = datetime.datetime(1900,1,1)
        self.filefmt = "*********%Y%m%d******"

    def setup(self, **kwargs):
        
        self.set_kwargs(**kwargs)
        
        if self.path is None:
            raise ValueError("The path is not valid")

        if self.online:
            log.log("Searching files in online mode...", self.name)

            for nTries in range(self.nTries):
                fullpath = self.searchFilesOnLine(self.path, self.startDate,
                    self.endDate, self.expLabel, self.ext, self.walk, 
                    self.filefmt, self.folderfmt)
                try:
                    fullpath = next(fullpath)
                except:
                    fullpath = None
                
                if fullpath:
                    self.fileSize = os.path.getsize(fullpath)
                    self.filename = fullpath
                    self.flagIsNewFile = 1
                    if self.fp != None:
                        self.fp.close()
                    self.fp = self.open_file(fullpath, self.open_mode)
                    self.flagNoMoreFiles = 0
                    break

                log.warning(
                    'Waiting {} sec for a valid file in {}: try {} ...'.format(
                        self.delay, self.path, nTries + 1), 
                    self.name)
                time.sleep(self.delay)

            if not(fullpath):
                raise schainpy.admin.SchainError(
                    'There isn\'t any valid file in {}'.format(self.path))            
            self.readFirstHeader()
        else:
            log.log("Searching files in {}".format(self.path), self.name)
            self.filenameList = self.searchFilesOffLine(self.path, self.startDate, 
                self.endDate, self.expLabel, self.ext, self.walk, self.filefmt, self.folderfmt)
            self.setNextFile()

    def checkForRealPath(self, nextFile, nextDay):
        '''
        '''

        dt = self.datatime + datetime.timedelta(1)
        filename = '{}.{}{}'.format(self.siteFile, dt.strftime('%Y%m%d'), self.ext)
        fullfilename = os.path.join(self.path, filename)
        if os.path.exists(fullfilename):
            return fullfilename, filename
        return None, filename
        
    
    def readFirstHeader(self):
        '''
        '''

        # 'peru2' ---> Piura  -   'peru1' ---> Huancayo or Porcuya
        self.siteFile = self.filename.split('/')[-1].split('.')[0]
        self.header_file = numpy.fromfile(self.fp, FILE_HEADER_STRUCTURE, 1)
        self.nrecords = self.header_file['nrec'][0]
        self.counter_records = 0
        self.flagIsNewFile = 0
        self.fileIndex += 1        

    def readNextBlock(self):

        while True:
            if not self.online and self.counter_records == self.nrecords:
                self.flagIsNewFile = 1
                if not self.setNextFile():
                    return 0
            try:
                pointer = self.fp.tell()
                self.readBlock()
            except:
                if self.online and self.waitDataBlock(pointer, 38512) == 1:
                    continue
                else:
                    if not self.setNextFile():
                        return 0

            if (self.datatime < datetime.datetime.combine(self.startDate, self.startTime)) or \
               (self.datatime > datetime.datetime.combine(self.endDate, self.endTime)):
                log.warning(
                    'Reading Record No. {}/{} -> {} [Skipping]'.format(
                        self.counter_records,
                        self.nrecords,
                        self.datatime.ctime()),
                    'BLTRParamReader')
                continue
            break

        log.log('Reading Record No. {} -> {}'.format(
            self.counter_records,
            self.datatime.ctime()), 'BLTRParamReader')

        return 1

    def readBlock(self):

        pointer = self.fp.tell()
        header_rec = numpy.fromfile(self.fp, REC_HEADER_STRUCTURE, 1)
        self.nchannels = int(header_rec['nchan'][0] / 2)
        self.kchan = header_rec['nrxs'][0]
        self.nmodes = header_rec['nmodes'][0]
        self.nranges = header_rec['nranges'][0]
        self.fp.seek(pointer)
        self.height = numpy.empty((self.nmodes, self.nranges))
        self.snr = numpy.empty((self.nmodes, int(self.nchannels), self.nranges))
        self.buffer = numpy.empty((self.nmodes, 3, self.nranges))
        self.flagDiscontinuousBlock = 0

        for mode in range(self.nmodes):
            self.readHeader()
            data = self.readData()
            self.height[mode] = (data[0] - self.correction) / 1000.
            self.buffer[mode] = data[1]
            self.snr[mode] = data[2]

        self.counter_records = self.counter_records + self.nmodes

        return

    def readHeader(self):
        '''
        RecordHeader of BLTR rawdata file
        '''

        header_structure = numpy.dtype(
            REC_HEADER_STRUCTURE.descr + [
                ('antenna_coord', 'f4', (2, int(self.nchannels))),
                ('rx_gains', 'u4', (int(self.nchannels),)),
                ('rx_analysis', 'u4', (int(self.nchannels),))
            ]
        )

        self.header_rec = numpy.fromfile(self.fp, header_structure, 1)
        self.lat = self.header_rec['lat'][0]
        self.lon = self.header_rec['lon'][0]
        self.delta = self.header_rec['delta_r'][0]
        self.correction = self.header_rec['dmode_rngcorr'][0]
        self.imode = self.header_rec['dmode_index'][0]
        self.antenna = self.header_rec['antenna_coord']
        self.rx_gains = self.header_rec['rx_gains']        
        self.time = self.header_rec['time'][0]               
        dt = datetime.datetime.utcfromtimestamp(self.time)
        if dt.date()>self.datatime.date():
            self.flagDiscontinuousBlock = 1
        self.datatime = dt
        
    def readData(self):
        '''
        Reading and filtering data block record of BLTR rawdata file, 
        filtering is according to status_value.

        Input:
            status_value - Array data is set to NAN for values that are not 
            equal to status_value

        '''
        self.nchannels = int(self.nchannels)

        data_structure = numpy.dtype(
            DATA_STRUCTURE.descr + [
                ('rx_saturation', 'u4', (self.nchannels,)),
                ('chan_offset', 'u4', (2 * self.nchannels,)),
                ('rx_amp', 'u4', (self.nchannels,)),
                ('rx_snr', 'f4', (self.nchannels,)),
                ('cross_snr', 'f4', (self.kchan,)),
                ('sea_power_relative', 'f4', (self.kchan,))]
        )

        data = numpy.fromfile(self.fp, data_structure, self.nranges)

        height = data['range']
        winds = numpy.array(
            (data['zonal'], data['meridional'], data['vertical']))
        snr = data['rx_snr'].T

        winds[numpy.where(winds == -9999.)] = numpy.nan
        winds[:, numpy.where(data['status'] != self.status_value)] = numpy.nan
        snr[numpy.where(snr == -9999.)] = numpy.nan
        snr[:, numpy.where(data['status'] != self.status_value)] = numpy.nan
        snr = numpy.power(10, snr / 10)

        return height, winds, snr

    def set_output(self):
        '''
        Storing data from databuffer to dataOut object
        '''

        self.dataOut.data_SNR = self.snr
        self.dataOut.height = self.height
        self.dataOut.data = self.buffer
        self.dataOut.utctimeInit = self.time
        self.dataOut.utctime = self.dataOut.utctimeInit
        self.dataOut.useLocalTime = False
        self.dataOut.paramInterval = 157
        self.dataOut.site = self.siteFile
        self.dataOut.nrecords = self.nrecords / self.nmodes
        self.dataOut.lat = self.lat
        self.dataOut.lon = self.lon
        self.dataOut.channelList = list(range(self.nchannels))
        self.dataOut.kchan = self.kchan        
        self.dataOut.delta = self.delta
        self.dataOut.correction = self.correction
        self.dataOut.nmodes = self.nmodes
        self.dataOut.imode = self.imode
        self.dataOut.antenna = self.antenna
        self.dataOut.rx_gains = self.rx_gains
        self.dataOut.flagNoData = False
        self.dataOut.flagDiscontinuousBlock = self.flagDiscontinuousBlock

    def getData(self):
        '''
        Storing data from databuffer to dataOut object
        '''
        if self.flagNoMoreFiles:
            self.dataOut.flagNoData = True
            return 0

        if not self.readNextBlock():
            self.dataOut.flagNoData = True
            return 0

        self.set_output()

        return 1
        
    def run(self, **kwargs):
        '''
        '''

        if not(self.isConfig):
            self.setup(**kwargs)
            self.isConfig = True

        self.getData()

        return


import numpy
import copy

class Beam:
    def __init__(self):
        self.codeList = []
        self.azimuthList = []
        self.zenithList = [] 


class AMISR:
    def __init__(self):
        self.flagNoData = True
        self.data = None
        self.utctime = None
        self.type = "AMISR"
        
        #propiedades para compatibilidad con Voltages
        self.timeZone = 0#timezone like jroheader, difference in minutes between UTC and localtime 
        self.dstFlag = 0#self.dataIn.dstFlag
        self.errorCount = 0#self.dataIn.errorCount
        self.useLocalTime = True#self.dataIn.useLocalTime
        
        self.radarControllerHeaderObj = None#self.dataIn.radarControllerHeaderObj.copy()
        self.systemHeaderObj = None#self.dataIn.systemHeaderObj.copy()
        self.channelList = [0]#self.dataIn.channelList esto solo aplica para el caso de AMISR
        self.dtype = numpy.dtype([('real','<f4'),('imag','<f4')])
        
        self.flagDiscontinuousBlock = None#self.dataIn.flagDiscontinuousBlock
        #self.utctime = #self.firstdatatime
        self.flagDecodeData = None#self.dataIn.flagDecodeData #asumo q la data esta decodificada
        self.flagDeflipData = None#self.dataIn.flagDeflipData #asumo q la data esta sin flip

        self.nCohInt = 1#self.dataIn.nCohInt
        self.nIncohInt = 1
        self.ippSeconds = None#self.dataIn.ippSeconds, segun el filename/Setup/Tufile
        self.windowOfFilter = None#self.dataIn.windowOfFilter
        
        self.timeInterval = None#self.dataIn.timeInterval*self.dataOut.nFFTPoints*self.dataOut.nIncohInt
        self.frequency = None#self.dataIn.frequency
        self.realtime = 0#self.dataIn.realtime
        
        #actualizar en la lectura de datos
        self.heightList = None#self.dataIn.heightList
        self.nProfiles = None#Number of samples or nFFTPoints
        self.nRecords = None
        self.nBeams = None
        self.nBaud = None#self.dataIn.nBaud
        self.nCode = None#self.dataIn.nCode
        self.code = None#self.dataIn.code
        
        #consideracion para los Beams
        self.beamCodeDict = None
        self.beamRangeDict = None
        self.beamcode = None
        self.azimuth = None
        self.zenith = None
        self.gain = None
    
        self.npulseByFrame = None
        
        self.profileIndex = None
        
        self.beam = Beam()
        
    def copy(self, inputObj=None):
        
        if inputObj is None:
            return copy.deepcopy(self)

        for key in list(inputObj.__dict__.keys()):
            self.__dict__[key] = inputObj.__dict__[key]
    
    def getNHeights(self):

        return len(self.heightList)

        
    def isEmpty(self):
        
        return self.flagNoData

    def getTimeInterval(self):
        
        timeInterval = self.ippSeconds * self.nCohInt
        
        return timeInterval
    
    timeInterval = property(getTimeInterval, "I'm the 'timeInterval' property")
    nHeights = property(getNHeights, "I'm the 'nHeights' property.")


from schainpy.model.data.jrodata import *
from schainpy.model.proc.jroproc_base import ProcessingUnit, Operation
from schainpy.model.io.jroIO_base import *

import scipy.io as sio 
import pprint
import numpy as np
from os import listdir
from os.path import isfile, join
import datetime
import cmath

class matoffReader(ProcessingUnit):
    
    index=None
    list=None
    firsttime=True
    utccounter=None
    utcfiletime=None
    utcmatcounter=0
    utcfirst=None
    utclist=None
    
    def __init__(self):
        self.dataOut = Spectra()
        self.online = False
        return
    
    def __setHeader(self, datastuff):
        
        self.dataOut.pairsList=[(0,1)]
        self.dataOut.channelList =  list(range(np.array(datastuff.get('power')).shape[1]))
        self.dataOut.nProfiles = len(np.array(datastuff.get('vel')).flatten())     #this!
        self.dataOut.nIncohInt = 20
        self.dataOut.nCohInt = 1 #this!
        self.dataOut.ippSeconds = 0.004 #this!
        self.dataOut.nFFTPoints = len(np.array(datastuff.get('vel')).flatten())    
        self.dataOut.timeZone = 0
        self.dataOut.heightList = np.array(datastuff.get('hts')).flatten()
    
    def __readFile(self, currentfile):
        print("Reading from this file:" + currentfile)
        
        #filesplit=currentfile.split("\\")
        filesplit=currentfile.split("/")
        newsplit=filesplit[-2]
        newnewsplit=newsplit.split(".")
        newnewsplit=[int(i) for i in newnewsplit]
        gooblist=datetime.datetime(newnewsplit[0],newnewsplit[1],newnewsplit[2],newnewsplit[3],newnewsplit[4],newnewsplit[5])
        self.utcfirst=(gooblist-datetime.datetime(1970,1,1)).total_seconds()
        
        
        newsplit=filesplit[-1]
        newnewsplit=newsplit.split(".")
        goobnum=newnewsplit[0]
        goobnum=int(goobnum)
        
        self.utcfirst=self.utcfirst+goobnum*2
#         if (currentfile[43:]=='0.mat'):
#             self.utcmatcounter=0
#             self.utcfirst=self.utclist[self.index]
            
#         if (self.utcmatcounter>60):
#             self.utcmatcounter=0
       
#         print self.utcmatcounter
        print(self.utcfirst)
        try:
            datastuff=sio.loadmat(currentfile)
        except:
            return None, None
        
        dataphase=datastuff.get('phase')
        data3=datastuff.get('doppler0')
        data4=datastuff.get('doppler1')
        data3= np.array(data3)
        data4 = np.array(data4)
        datacoh=datastuff.get('coherence2')
        
        datacohphase=datacoh*np.exp(-dataphase*1j)
#         data31 = np.fliplr(data3)
#         data41 = np.fliplr(data4)
          
        data31 = data3.reshape((1,data3.shape[0],data3.shape[1]))
        data41 = data4.reshape((1,data4.shape[0],data4.shape[1]))
        datacohphase1 = datacohphase.reshape((1,datacoh.shape[0],datacoh.shape[1]))
        
        datastack = np.vstack((data31,data41))
        
        self.__setHeader(datastuff)
        
        spc = datastack
        cspc = datacohphase1
        
        return spc, cspc
    
    def __findFiles(self, path, startDate=None, endDate=None,startTime=datetime.time(0,0,0), endTime=datetime.time(23,59,59)):
        
        if startDate == None:
            startDate = datetime.date(1970,1,1)
            
        if endDate == None:
            endDate = datetime.date(2050,1,1)
            
        startsearch1=datetime.datetime.combine(startDate,startTime)
        startsearch2=(startsearch1-datetime.datetime(1970,1,1)).total_seconds()
        endsearch1=datetime.datetime.combine(endDate,endTime)
        endsearch2=(endsearch1-datetime.datetime(1970,1,1)).total_seconds()
        
        dirList = listdir(path)
        dirList = sorted(dirList)
        
        dirListFiltered=[]
        fileListFiltered=[]
        utclist=[]
        
        if not dirList:
            print("No directories found")
            return []
        
        #if self.online:
        #    dirList= [dirList[-1]]
         
        if self.online:
            currentdate = datetime.datetime.now()
            strsplit1=currentdate.strftime('%Y.%m.%d')
            dirList = fnmatch.filter(dirList,strsplit1+'*')  
        
        for thisDir in dirList:
            if not os.path.isdir(os.path.join(path, thisDir)):
                continue
            
            strsplit=thisDir.split('.')
            timeints=[int(i) for i in strsplit]
            timelist=datetime.datetime(timeints[0],timeints[1],timeints[2],timeints[3],timeints[4],timeints[5])
            utctime=(timelist-datetime.datetime(1970,1,1)).total_seconds()
            
            if not self.online:
                if (utctime > endsearch2):
                    continue
                
                if (utctime < startsearch2):
                    continue
            
            dirListFiltered.append(thisDir)
            utclist.append(utctime)
        
        if not dirListFiltered:
            print("filtro")
            return []
        
        for thisDir in dirListFiltered:
            
            pathFile = os.path.join(self.path, thisDir)
            
            fileList = os.listdir(pathFile)
            
            if not fileList:
                continue
            
            for k in range(len(fileList)):
                thisFile = str(k)+'.mat'
                
                if not os.path.isfile(os.path.join(pathFile, thisFile)):
                    continue
                
                fileListFiltered.append(os.path.join(pathFile, thisFile))
                    
        return fileListFiltered
    
    def __getNextOnlineFile(self, seconds = 40):
        
        filename = self.__getNextOfflineFile()
        
        if filename:
            return filename
        
        ncurrentfiles = len(self.fileList)
        
        nTries = 0
        while (True):
            filelist = self.__findFiles(self.path)
            if len(filelist) > ncurrentfiles:
                break

            nTries += 1
            
            if nTries > 3:
                break
            
            print("Waiting %d seconds ..." %seconds)
            time.sleep(40)
        
        if not (len(filelist) > ncurrentfiles):
            return None
        
        self.fileList = filelist
        filename = self.__getNextOfflineFile()
        
        return filename
    
    def __getNextOfflineFile(self):
        
        if self.index >= len(self.fileList):
            return None
         
        filename=self.fileList[self.index]
        self.index += 1
        return filename
    
    def __getNextFile(self):
        
        if self.online:
            filename = self.__getNextOnlineFile()
        else:
            filename = self.__getNextOfflineFile()   
        return filename
    
    def setup(self, path, startDate=None, endDate=None,startTime=datetime.time(0,0,0), endTime=datetime.time(23,59,59)):
        
        fileList = self.__findFiles(path, startDate, endDate, startTime, endTime)
        
        if self.online:
            self.index = len(fileList) -1
        else:
            self.index = 0
            
        self.fileList = fileList
        
        print("fin setup")
    
    def run(self,path=None,startDate=None, endDate=None,
            startTime=datetime.time(0,0,0),
            endTime=datetime.time(23,59,59),
            walk=True,timezone='ut',
            all=0,online=False,ext=None,**kwargs):
        
        self.path=path
        self.ext=ext
        self.startDate=startDate
        self.endDate=endDate
        self.startTime=startTime
        self.endTime=endTime
        self.online = online
        self.dataOut.flagNoData = True  
        
        if (self.firsttime==True):
            self.setup(path, startDate, endDate, startTime, endTime)      
            self.firsttime=False
            
        
        if not self.fileList:
            self.dataOut.flagNoData = True
            print("lista vacia")
            return
        
        currentfile = self.__getNextFile()
        
        if not currentfile:
            "no file"
            return
        
        spc, cspc = self.__readFile(currentfile)
        
        if spc!=None:
            
            self.dataOut.data_spc = spc
            self.dataOut.data_cspc = cspc
            self.dataOut.utctime = self.utcfirst
            self.dataOut.flagNoData = False  
  
        return 1
 


import numpy,math,random,time
#---------------1  Heredamos JRODatareader
from schainpy.model.io.jroIO_base import *
#---------------2 Heredamos las propiedades de ProcessingUnit
from schainpy.model.proc.jroproc_base import ProcessingUnit,Operation,MPDecorator
#---------------3 Importaremos las clases BascicHeader, SystemHeader, RadarControlHeader, ProcessingHeader
from schainpy.model.data.jroheaderIO import PROCFLAG, BasicHeader,SystemHeader,RadarControllerHeader, ProcessingHeader
#---------------4 Importaremos el objeto Voltge
from schainpy.model.data.jrodata import Voltage

class SimulatorReader(JRODataReader, ProcessingUnit):
    incIntFactor                   = 1
    nFFTPoints                     = 0
    FixPP_IncInt                   = 1
    FixRCP_IPP                     = 1000
    FixPP_CohInt                   = 1
    Tau_0                          = 250
    AcqH0_0                        = 70
    H0                             = AcqH0_0
    AcqDH_0                        = 1.25
    DH0                            = AcqDH_0
    Bauds                          = 32
    BaudWidth                      = None
    FixRCP_TXA                     = 40
    FixRCP_TXB                     = 70
    fAngle                         = 2.0*math.pi*(1/16)
    DC_level                       = 500
    stdev                          = 8
    Num_Codes                      = 2
    #code0                          = numpy.array([1,1,1,0,1,1,0,1,1,1,1,0,0,0,1,0,1,1,1,0,1,1,0,1,0,0,0,1,1,1,0,1])
    #code1                          = numpy.array([1,1,1,0,1,1,0,1,1,1,1,0,0,0,1,0,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0])
    #Dyn_snCode                     = numpy.array([Num_Codes,Bauds])
    Dyn_snCode                     = None
    Samples                        = 200
    channels                       = 2
    pulses                         = None
    Reference                      = None
    pulse_size                     = None
    prof_gen                       = None
    Fdoppler                       = 100
    Hdoppler                       = 36
    Adoppler                       = 300
    frequency                      = 9345
    nTotalReadFiles                = 1000

    def __init__(self):
        """
        Inicializador de la clases SimulatorReader para
        generar datos de voltage simulados.
        Input:
            dataOut: Objeto de la clase Voltage.
            Este Objeto sera utilizado apra almacenar
            un perfil de datos  cada vez qe se haga
            un requerimiento (getData)
        """
        ProcessingUnit.__init__(self)
        print(" [ START ] init - Metodo Simulator Reader")

        self.isConfig                  = False
        self.basicHeaderObj            = BasicHeader(LOCALTIME)
        self.systemHeaderObj           = SystemHeader()
        self.radarControllerHeaderObj  = RadarControllerHeader()
        self.processingHeaderObj       = ProcessingHeader()
        self.profileIndex              = 2**32-1
        self.dataOut                   = Voltage()
        #code0                          = numpy.array([1,1,1,0,1,1,0,1,1,1,1,0,0,0,1,0,1,1,1,0,1,1,0,1,0,0,0,1,1,1,0,1])
        code0                          = numpy.array([1,1,1,-1,1,1,-1,1,1,1,1,-1,-1,-1,1,-1,1,1,1,-1,1,1,-1,1,-1,-1,-1,1,1,1,-1,1])
        #code1                          = numpy.array([1,1,1,0,1,1,0,1,1,1,1,0,0,0,1,0,0,0,0,1,0,0,1,0,1,1,1,0,0,0,1,0])
        code1                          = numpy.array([1,1,1,-1,1,1,-1,1,1,1,1,-1,-1,-1,1,-1,-1,-1,-1,1,-1,-1,1,-1,1,1,1,-1,-1,-1,1,-1])
        #self.Dyn_snCode                = numpy.array([code0,code1])
        self.Dyn_snCode                = None

    def set_kwargs(self, **kwargs):
        for key, value in kwargs.items():
            setattr(self, key, value)

    def __hasNotDataInBuffer(self):

        if self.profileIndex >= self.processingHeaderObj.profilesPerBlock* self.nTxs:
            if self.nReadBlocks>0:
                tmp                           = self.dataOut.utctime
                tmp_utc                       = int(self.dataOut.utctime)
                tmp_milisecond                = int((tmp-tmp_utc)*1000)
                self.basicHeaderObj.utc       = tmp_utc
                self.basicHeaderObj.miliSecond= tmp_milisecond
            return 1
        return 0

    def setNextFile(self):
        """Set the next file to be readed open it and parse de file header"""

        if (self.nReadBlocks >= self.processingHeaderObj.dataBlocksPerFile):
            self.nReadFiles=self.nReadFiles+1
            if self.nReadFiles > self.nTotalReadFiles:
                self.flagNoMoreFiles=1
                raise schainpy.admin.SchainWarning('No more files to read')

            print('------------------- [Opening file] ------------------------------',self.nReadFiles)
            self.nReadBlocks  = 0
        #if self.nReadBlocks==0:
        #    self.readFirstHeader()

    def __setNewBlock(self):
        self.setNextFile()
        if self.flagIsNewFile:
            return 1

    def readNextBlock(self):
        while True:
            self.__setNewBlock()
            if not(self.readBlock()):
                return 0
            self.getBasicHeader()
            break
        if self.verbose:
            print("[Reading] Block No. %d/%d -> %s" %(self.nReadBlocks,
                                                      self.processingHeaderObj.dataBlocksPerFile,
                                                      self.dataOut.datatime.ctime()) )
        return 1

    def getFirstHeader(self):
        self.getBasicHeader()
        self.dataOut.processingHeaderObj      = self.processingHeaderObj.copy()
        self.dataOut.systemHeaderObj          = self.systemHeaderObj.copy()
        self.dataOut.radarControllerHeaderObj = self.radarControllerHeaderObj.copy()
        self.dataOut.dtype       = self.dtype

        self.dataOut.nProfiles   = self.processingHeaderObj.profilesPerBlock
        self.dataOut.heightList  = numpy.arange(self.processingHeaderObj.nHeights) * self.processingHeaderObj.deltaHeight + self.processingHeaderObj.firstHeight
        self.dataOut.channelList = list(range(self.systemHeaderObj.nChannels))
        self.dataOut.nCohInt     = self.processingHeaderObj.nCohInt
        # asumo q la data no esta decodificada
        self.dataOut.flagDecodeData  = self.processingHeaderObj.flag_decode
        # asumo q la data no esta sin flip
        self.dataOut.flagDeflipData  = self.processingHeaderObj.flag_deflip
        self.dataOut.flagShiftFFT    = self.processingHeaderObj.shif_fft
        self.dataOut.frequency       = self.frequency

    def getBasicHeader(self):
        self.dataOut.utctime = self.basicHeaderObj.utc + self.basicHeaderObj.miliSecond / \
            1000. + self.profileIndex * self.radarControllerHeaderObj.ippSeconds

        self.dataOut.flagDiscontinuousBlock = self.flagDiscontinuousBlock
        self.dataOut.timeZone               = self.basicHeaderObj.timeZone
        self.dataOut.dstFlag                = self.basicHeaderObj.dstFlag
        self.dataOut.errorCount             = self.basicHeaderObj.errorCount
        self.dataOut.useLocalTime           = self.basicHeaderObj.useLocalTime
        self.dataOut.ippSeconds             = self.radarControllerHeaderObj.ippSeconds / self.nTxs

    def readFirstHeader(self):

        datatype = int(numpy.log2((self.processingHeaderObj.processFlags &
                                   PROCFLAG.DATATYPE_MASK)) - numpy.log2(PROCFLAG.DATATYPE_CHAR))
        if datatype == 0:
            datatype_str = numpy.dtype([('real', '<i1'), ('imag', '<i1')])
        elif datatype == 1:
            datatype_str = numpy.dtype([('real', '<i2'), ('imag', '<i2')])
        elif datatype == 2:
            datatype_str = numpy.dtype([('real', '<i4'), ('imag', '<i4')])
        elif datatype == 3:
            datatype_str = numpy.dtype([('real', '<i8'), ('imag', '<i8')])
        elif datatype == 4:
            datatype_str = numpy.dtype([('real', '<f4'), ('imag', '<f4')])
        elif datatype == 5:
            datatype_str = numpy.dtype([('real', '<f8'), ('imag', '<f8')])
        else:
            raise ValueError('Data type was not defined')

        self.dtype = datatype_str


    def set_RCH(self, expType=2, nTx=1,ipp=None, txA=0, txB=0,
                 nWindows=None, nHeights=None, firstHeight=None, deltaHeight=None,
                 numTaus=0, line6Function=0, line5Function=0, fClock=None,
                 prePulseBefore=0, prePulseAfter=0,
                 codeType=0, nCode=0, nBaud=0, code=None,
                 flip1=0, flip2=0,Taus=0):
        self.radarControllerHeaderObj.expType       = expType
        self.radarControllerHeaderObj.nTx           = nTx
        self.radarControllerHeaderObj.ipp           = float(ipp)
        self.radarControllerHeaderObj.txA           = float(txA)
        self.radarControllerHeaderObj.txB           = float(txB)
        self.radarControllerHeaderObj.rangeIpp      = b'A\n'#ipp
        self.radarControllerHeaderObj.rangeTxA      = b''
        self.radarControllerHeaderObj.rangeTxB      = b''

        self.radarControllerHeaderObj.nHeights      = int(nHeights)
        self.radarControllerHeaderObj.firstHeight   = numpy.array([firstHeight])
        self.radarControllerHeaderObj.deltaHeight   = numpy.array([deltaHeight])
        self.radarControllerHeaderObj.samplesWin    = numpy.array([nHeights])


        self.radarControllerHeaderObj.nWindows      = nWindows
        self.radarControllerHeaderObj.numTaus       = numTaus
        self.radarControllerHeaderObj.codeType      = codeType
        self.radarControllerHeaderObj.line6Function = line6Function
        self.radarControllerHeaderObj.line5Function = line5Function
        #self.radarControllerHeaderObj.fClock        = fClock
        self.radarControllerHeaderObj.prePulseBefore= prePulseBefore
        self.radarControllerHeaderObj.prePulseAfter = prePulseAfter

        self.radarControllerHeaderObj.flip1         = flip1
        self.radarControllerHeaderObj.flip2         = flip2

        self.radarControllerHeaderObj.code_size     = 0
        if self.radarControllerHeaderObj.codeType  != 0:
            self.radarControllerHeaderObj.nCode         = nCode
            self.radarControllerHeaderObj.nBaud         = nBaud
            self.radarControllerHeaderObj.code          = code
            self.radarControllerHeaderObj.code_size     = int(numpy.ceil(nBaud / 32.)) * nCode * 4

        if fClock is None and deltaHeight is not None:
            self.fClock = 0.15 / (deltaHeight * 1e-6)
            self.radarControllerHeaderObj.fClock     = self.fClock
        if numTaus==0:
            self.radarControllerHeaderObj.Taus       =  numpy.array(0,'<f4')
        else:
            self.radarControllerHeaderObj.Taus       = numpy.array(Taus,'<f4')

    def set_PH(self, dtype=0, blockSize=0, profilesPerBlock=0,
                  dataBlocksPerFile=0, nWindows=0, processFlags=0, nCohInt=0,
                  nIncohInt=0, totalSpectra=0, nHeights=0, firstHeight=0,
                  deltaHeight=0, samplesWin=0, spectraComb=0, nCode=0,
                  code=0, nBaud=None, shif_fft=False, flag_dc=False,
                  flag_cspc=False, flag_decode=False, flag_deflip=False):

        self.processingHeaderObj.dtype             = dtype
        self.processingHeaderObj.profilesPerBlock  = profilesPerBlock
        self.processingHeaderObj.dataBlocksPerFile = dataBlocksPerFile
        self.processingHeaderObj.nWindows          = nWindows
        self.processingHeaderObj.processFlags      = processFlags
        self.processingHeaderObj.nCohInt           = nCohInt
        self.processingHeaderObj.nIncohInt         = nIncohInt
        self.processingHeaderObj.totalSpectra      = totalSpectra

        self.processingHeaderObj.nHeights          = int(nHeights)
        self.processingHeaderObj.firstHeight       = firstHeight#numpy.array([firstHeight])#firstHeight
        self.processingHeaderObj.deltaHeight       = deltaHeight#numpy.array([deltaHeight])#deltaHeight
        self.processingHeaderObj.samplesWin        = nHeights#numpy.array([nHeights])#nHeights

    def set_BH(self, utc = 0, miliSecond = 0, timeZone = 0):
        self.basicHeaderObj.utc                    = utc
        self.basicHeaderObj.miliSecond             = miliSecond
        self.basicHeaderObj.timeZone               = timeZone

    def set_SH(self, nSamples=0, nProfiles=0, nChannels=0, adcResolution=14, pciDioBusWidth=32):
        #self.systemHeaderObj.size           = size
        self.systemHeaderObj.nSamples       = nSamples
        self.systemHeaderObj.nProfiles      = nProfiles
        self.systemHeaderObj.nChannels      = nChannels
        self.systemHeaderObj.adcResolution  = adcResolution
        self.systemHeaderObj.pciDioBusWidth = pciDioBusWidth

    def init_acquisition(self):

        if self.nFFTPoints != 0:
            self.incIntFactor = m_nProfilesperBlock/self.nFFTPoints
            if (self.FixPP_IncInt > self.incIntFactor):
                self.incIntFactor = self.FixPP_IncInt/ self.incIntFactor
            elif(self.FixPP_IncInt< self.incIntFactor):
                print("False alert...")

        ProfilesperBlock  = self.processingHeaderObj.profilesPerBlock

        self.timeperblock =int(((self.FixRCP_IPP
                        *ProfilesperBlock
                        *self.FixPP_CohInt
                        *self.incIntFactor)
                       /150.0)
                      *0.9
                      +0.5)
        # para cada canal
        self.profiles     =  ProfilesperBlock*self.FixPP_CohInt
        self.profiles     =  ProfilesperBlock
        self.Reference    =  int((self.Tau_0-self.AcqH0_0)/(self.AcqDH_0)+0.5)
        self.BaudWidth    =  int((self.FixRCP_TXA/self.AcqDH_0)/self.Bauds + 0.5 )

        if (self.BaudWidth==0):
            self.BaudWidth=1

    def init_pulse(self,Num_Codes=Num_Codes,Bauds=Bauds,BaudWidth=BaudWidth,Dyn_snCode=Dyn_snCode):

        Num_Codes   = Num_Codes
        Bauds       = Bauds
        BaudWidth   = BaudWidth
        Dyn_snCode  = Dyn_snCode

        if Dyn_snCode:
            print("EXISTE")
        else:
            print("No existe")

        if Dyn_snCode: # if Bauds:
            pulses         = list(range(0,Num_Codes))
            num_codes      = Num_Codes
            for i in range(num_codes):
                pulse_size = Bauds*BaudWidth
                pulses[i]  = numpy.zeros(pulse_size)
                for j in range(Bauds):
                    for k in range(BaudWidth):
                        pulses[i][j*BaudWidth+k] = int(Dyn_snCode[i][j]*600)
        else:
            print("sin code")
            pulses     = list(range(1))
            if self.AcqDH_0>0.149:
                pulse_size = int(self.FixRCP_TXB/0.15+0.5)
            else:
                pulse_size = int((self.FixRCP_TXB/self.AcqDH_0)+0.5) #0.0375
            pulses[0]  = numpy.ones(pulse_size)
            pulses     = 600*pulses[0]

        return pulses,pulse_size

    def jro_GenerateBlockOfData(self,Samples=Samples,DC_level= DC_level,stdev=stdev,
                                Reference= Reference,pulses= pulses,
                                Num_Codes= Num_Codes,pulse_size=pulse_size,
                                prof_gen= prof_gen,H0 = H0,DH0=DH0,
                                Adoppler=Adoppler,Fdoppler= Fdoppler,Hdoppler=Hdoppler):
        Samples    = Samples
        DC_level   = DC_level
        stdev      = stdev
        m_nR       = Reference
        pulses     = pulses
        num_codes  = Num_Codes
        ps         = pulse_size
        prof_gen   = prof_gen
        channels   = self.channels
        H0         = H0
        DH0        = DH0
        ippSec     = self.radarControllerHeaderObj.ippSeconds
        Fdoppler   = self.Fdoppler
        Hdoppler   = self.Hdoppler
        Adoppler   = self.Adoppler

        self.datablock = numpy.zeros([channels,prof_gen,Samples],dtype= numpy.complex64)
        for i in range(channels):
            for k in range(prof_gen):
                #·······················NOISE···············
                Noise_r    = numpy.random.normal(DC_level,stdev,Samples)
                Noise_i    = numpy.random.normal(DC_level,stdev,Samples)
                Noise      = numpy.zeros(Samples,dtype=complex)
                Noise.real = Noise_r
                Noise.imag = Noise_i
                #·······················PULSOS··············
                Pulso      = numpy.zeros(pulse_size,dtype=complex)
                Pulso.real =  pulses[k%num_codes]
                Pulso.imag =  pulses[k%num_codes]
                #····················· PULSES+NOISE··········
                InBuffer                    = numpy.zeros(Samples,dtype=complex)
                InBuffer[m_nR:m_nR+ps]      = Pulso
                InBuffer                    =  InBuffer+Noise
                #····················· ANGLE ·······························
                InBuffer.real[m_nR:m_nR+ps] = InBuffer.real[m_nR:m_nR+ps]*(math.cos( self.fAngle)*5)
                InBuffer.imag[m_nR:m_nR+ps] = InBuffer.imag[m_nR:m_nR+ps]*(math.sin( self.fAngle)*5)
                InBuffer=InBuffer
                self.datablock[i][k]= InBuffer

        #················DOPPLER SIGNAL...............................................
        time_vec   = numpy.linspace(0,(prof_gen-1)*ippSec,int(prof_gen))+self.nReadBlocks*ippSec*prof_gen+(self.nReadFiles-1)*ippSec*prof_gen
        fd         = Fdoppler #+(600.0/120)*self.nReadBlocks
        d_signal   = Adoppler*numpy.array(numpy.exp(1.0j*2.0*math.pi*fd*time_vec),dtype=numpy.complex64)
        #·············Señal con ancho espectral····················
        if prof_gen%2==0:
            min = int(prof_gen/2.0-1.0)
            max = int(prof_gen/2.0)
        else:
            min = int(prof_gen/2.0)
            max = int(prof_gen/2.0)
        specw_sig  = numpy.linspace(-min,max,prof_gen)
        w          = 4
        A          = 20
        specw_sig   = specw_sig/w
        specw_sig   = numpy.sinc(specw_sig)
        specw_sig   =  A*numpy.array(specw_sig,dtype=numpy.complex64)
        #·················· DATABLOCK + DOPPLER····················
        HD=int(Hdoppler/self.AcqDH_0)
        for  i in range(12):
            self.datablock[0,:,HD+i]=self.datablock[0,:,HD+i]+ d_signal# RESULT
        #·················· DATABLOCK + DOPPLER*Sinc(x)····················
        HD=int(Hdoppler/self.AcqDH_0)
        HD=int(HD/2)
        for  i in range(12):
            self.datablock[0,:,HD+i]=self.datablock[0,:,HD+i]+ specw_sig*d_signal# RESULT

    def readBlock(self):

        self.jro_GenerateBlockOfData(Samples= self.samples,DC_level=self.DC_level,
                                     stdev=self.stdev,Reference= self.Reference,
                                     pulses = self.pulses,Num_Codes=self.Num_Codes,
                                     pulse_size=self.pulse_size,prof_gen=self.profiles,
                                     H0=self.H0,DH0=self.DH0)

        self.profileIndex   = 0
        self.flagIsNewFile  = 0
        self.flagIsNewBlock = 1
        self.nTotalBlocks  += 1
        self.nReadBlocks   += 1

        return 1


    def getData(self):
        if self.flagNoMoreFiles:
            self.dataOut.flagNodata = True
            return 0
        self.flagDiscontinuousBlock = 0
        self.flagIsNewBlock         = 0
        if self.__hasNotDataInBuffer():   # aqui es verdad
            if not(self.readNextBlock()): # return 1 y por eso el if not salta  a getBasic Header
                return 0
            self.getFirstHeader()         # atributo

        if not self.getByBlock:
            self.dataOut.flagDataAsBlock = False
            self.dataOut.data = self.datablock[:, self.profileIndex, :]
            self.dataOut.profileIndex = self.profileIndex
            self.profileIndex += 1
        else:
            pass
        self.dataOut.flagNoData = False
        self.getBasicHeader()
        self.dataOut.realtime = self.online
        return self.dataOut.data


    def setup(self,frequency=49.92e6,incIntFactor= 1, nFFTPoints = 0, FixPP_IncInt=1,FixRCP_IPP=1000,
                   FixPP_CohInt= 1,Tau_0= 250,AcqH0_0 = 70 ,AcqDH_0=1.25, Bauds= 32,
                   FixRCP_TXA = 40, FixRCP_TXB = 50, fAngle = 2.0*math.pi*(1/16),DC_level= 50,
                   stdev= 8,Num_Codes = 1 , Dyn_snCode = None, samples=200,
                   channels=2,Fdoppler=20,Hdoppler=36,Adoppler=500,
                   profilesPerBlock=300,dataBlocksPerFile=120,nTotalReadFiles=10000,
                   **kwargs):

        self.set_kwargs(**kwargs)
        self.nReadBlocks = 0
        self.nReadFiles  = 1
        print('------------------- [Opening file: ] ------------------------------',self.nReadFiles)

        tmp              = time.time()
        tmp_utc          = int(tmp)
        tmp_milisecond   = int((tmp-tmp_utc)*1000)
        print(" SETUP -basicHeaderObj.utc",datetime.datetime.utcfromtimestamp(tmp))
        if Dyn_snCode is None:
            Num_Codes=1
            Bauds    =1



        self.set_BH(utc= tmp_utc,miliSecond= tmp_milisecond,timeZone=300 )
        self.set_RCH( expType=0, nTx=150,ipp=FixRCP_IPP, txA=FixRCP_TXA, txB= FixRCP_TXB,
                 nWindows=1 , nHeights=samples, firstHeight=AcqH0_0, deltaHeight=AcqDH_0,
                 numTaus=1, line6Function=0, line5Function=0, fClock=None,
                 prePulseBefore=0, prePulseAfter=0,
                 codeType=0, nCode=Num_Codes, nBaud=32, code=Dyn_snCode,
                 flip1=0, flip2=0,Taus=Tau_0)

        self.set_PH(dtype=0, blockSize=0, profilesPerBlock=profilesPerBlock,
                      dataBlocksPerFile=dataBlocksPerFile, nWindows=1, processFlags=numpy.array([1024]), nCohInt=1,
                      nIncohInt=1, totalSpectra=0, nHeights=samples, firstHeight=AcqH0_0,
                      deltaHeight=AcqDH_0, samplesWin=samples, spectraComb=0, nCode=0,
                      code=0, nBaud=None, shif_fft=False, flag_dc=False,
                      flag_cspc=False, flag_decode=False, flag_deflip=False)

        self.set_SH(nSamples=samples, nProfiles=profilesPerBlock, nChannels=channels)

        self.readFirstHeader()

        self.frequency                      = frequency
        self.incIntFactor                   = incIntFactor
        self.nFFTPoints                     = nFFTPoints
        self.FixPP_IncInt                   = FixPP_IncInt
        self.FixRCP_IPP                     = FixRCP_IPP
        self.FixPP_CohInt                   = FixPP_CohInt
        self.Tau_0                          = Tau_0
        self.AcqH0_0                        = AcqH0_0
        self.H0                             = AcqH0_0
        self.AcqDH_0                        = AcqDH_0
        self.DH0                            = AcqDH_0
        self.Bauds                          = Bauds
        self.FixRCP_TXA                     = FixRCP_TXA
        self.FixRCP_TXB                     = FixRCP_TXB
        self.fAngle                         = fAngle
        self.DC_level                       = DC_level
        self.stdev                          = stdev
        self.Num_Codes                      = Num_Codes
        self.Dyn_snCode                     = Dyn_snCode
        self.samples                        = samples
        self.channels                       = channels
        self.profiles                       = None
        self.m_nReference                   = None
        self.Baudwidth                      = None
        self.Fdoppler                       = Fdoppler
        self.Hdoppler                       = Hdoppler
        self.Adoppler                       = Adoppler
        self.nTotalReadFiles                = int(nTotalReadFiles)

        print("IPP    ", self.FixRCP_IPP)
        print("Tau_0  ",self.Tau_0)
        print("AcqH0_0",self.AcqH0_0)
        print("samples,window ",self.samples)
        print("AcqDH_0",AcqDH_0)
        print("FixRCP_TXA",self.FixRCP_TXA)
        print("FixRCP_TXB",self.FixRCP_TXB)
        print("Dyn_snCode",Dyn_snCode)
        print("Fdoppler", Fdoppler)
        print("Hdoppler",Hdoppler)
        print("Vdopplermax",Fdoppler*(3.0e8/self.frequency)/2.0)
        print("nTotalReadFiles", nTotalReadFiles)

        self.init_acquisition()
        self.pulses,self.pulse_size=self.init_pulse(Num_Codes=self.Num_Codes,Bauds=self.Bauds,BaudWidth=self.BaudWidth,Dyn_snCode=Dyn_snCode)
        print(" [ END ] - SETUP metodo")
        return

    def run(self,**kwargs): # metodo propio
        if not(self.isConfig):
            self.setup(**kwargs)
            self.isConfig = True
        self.getData()



'''

$Author: murco $
$Id: JROHeaderIO.py 151 2012-10-31 19:00:51Z murco $
'''
import sys
import numpy
import copy
import datetime
import inspect
from schainpy.utils import log

SPEED_OF_LIGHT = 299792458
SPEED_OF_LIGHT = 3e8

BASIC_STRUCTURE = numpy.dtype([
    ('nSize', '<u4'),
    ('nVersion', '<u2'),
    ('nDataBlockId', '<u4'),
    ('nUtime', '<u4'),
    ('nMilsec', '<u2'),
    ('nTimezone', '<i2'),
    ('nDstflag', '<i2'),
    ('nErrorCount', '<u4')
])

SYSTEM_STRUCTURE = numpy.dtype([
    ('nSize', '<u4'),
    ('nNumSamples', '<u4'),
    ('nNumProfiles', '<u4'),
    ('nNumChannels', '<u4'),
    ('nADCResolution', '<u4'),
    ('nPCDIOBusWidth', '<u4'),
])

RADAR_STRUCTURE = numpy.dtype([
                             ('nSize', '<u4'),
                             ('nExpType', '<u4'),
                             ('nNTx', '<u4'),
                             ('fIpp', '<f4'),
                             ('fTxA', '<f4'),
                             ('fTxB', '<f4'),
                             ('nNumWindows', '<u4'),
                             ('nNumTaus', '<u4'),
                             ('nCodeType', '<u4'),
                             ('nLine6Function', '<u4'),
                             ('nLine5Function', '<u4'),
                             ('fClock', '<f4'),
                             ('nPrePulseBefore', '<u4'),
                             ('nPrePulseAfter', '<u4'),
                             ('sRangeIPP', '<a20'),
                             ('sRangeTxA', '<a20'),
                             ('sRangeTxB', '<a20'),
])

SAMPLING_STRUCTURE = numpy.dtype(
    [('h0', '<f4'), ('dh', '<f4'), ('nsa', '<u4')])


PROCESSING_STRUCTURE = numpy.dtype([
    ('nSize', '<u4'),
    ('nDataType', '<u4'),
    ('nSizeOfDataBlock', '<u4'),
    ('nProfilesperBlock', '<u4'),
    ('nDataBlocksperFile', '<u4'),
    ('nNumWindows', '<u4'),
    ('nProcessFlags', '<u4'),
    ('nCoherentIntegrations', '<u4'),
    ('nIncoherentIntegrations', '<u4'),
    ('nTotalSpectra', '<u4')
])


class Header(object):

    def __init__(self):
        raise NotImplementedError

    def copy(self):
        return copy.deepcopy(self)

    def read(self):

        raise NotImplementedError

    def write(self):

        raise NotImplementedError

    def getAllowedArgs(self):
        args = inspect.getargspec(self.__init__).args
        try:
            args.remove('self')
        except:
            pass
        return args

    def getAsDict(self):
        args = self.getAllowedArgs()
        asDict = {}
        for x in args:
            asDict[x] = self[x]
        return asDict

    def __getitem__(self, name):
        return getattr(self, name)

    def printInfo(self):

        message = "#" * 50 + "\n"
        message += self.__class__.__name__.upper() + "\n"
        message += "#" * 50 + "\n"

        keyList = list(self.__dict__.keys())
        keyList.sort()

        for key in keyList:
            message += "%s = %s" % (key, self.__dict__[key]) + "\n"

        if "size" not in keyList:
            attr = getattr(self, "size")

            if attr:
                message += "%s = %s" % ("size", attr) + "\n"

        print(message)


class BasicHeader(Header):

    size = None
    version = None
    dataBlock = None
    utc = None
    ltc = None
    miliSecond = None
    timeZone = None
    dstFlag = None
    errorCount = None
    datatime = None
    structure = BASIC_STRUCTURE
    __LOCALTIME = None

    def __init__(self, useLocalTime=True):

        self.size = 24
        self.version = 0
        self.dataBlock = 0
        self.utc = 0
        self.miliSecond = 0
        self.timeZone = 0
        self.dstFlag = 0
        self.errorCount = 0

        self.useLocalTime = useLocalTime

    def read(self, fp):

        self.length = 0
        try:
            if hasattr(fp, 'read'):
                header = numpy.fromfile(fp, BASIC_STRUCTURE, 1)
            else:
                header = numpy.fromstring(fp, BASIC_STRUCTURE, 1)
        except Exception as e:
            print("BasicHeader: ")
            print(e)
            return 0

        self.size = int(header['nSize'][0])
        self.version = int(header['nVersion'][0])
        self.dataBlock = int(header['nDataBlockId'][0])
        self.utc = int(header['nUtime'][0])
        self.miliSecond = int(header['nMilsec'][0])
        self.timeZone = int(header['nTimezone'][0])
        self.dstFlag = int(header['nDstflag'][0])
        self.errorCount = int(header['nErrorCount'][0])

        if self.size < 24:
            return 0

        self.length = header.nbytes
        return 1

    def write(self, fp):

        headerTuple = (self.size, self.version, self.dataBlock, self.utc,
                       self.miliSecond, self.timeZone, self.dstFlag, self.errorCount)
        header = numpy.array(headerTuple, BASIC_STRUCTURE)
        header.tofile(fp)

        return 1

    def get_ltc(self):

        return self.utc - self.timeZone * 60

    def set_ltc(self, value):

        self.utc = value + self.timeZone * 60

    def get_datatime(self):

        return datetime.datetime.utcfromtimestamp(self.ltc)

    ltc = property(get_ltc, set_ltc)
    datatime = property(get_datatime)


class SystemHeader(Header):

    size = None
    nSamples = None
    nProfiles = None
    nChannels = None
    adcResolution = None
    pciDioBusWidth = None
    structure = SYSTEM_STRUCTURE

    def __init__(self, nSamples=0, nProfiles=0, nChannels=0, adcResolution=14, pciDioBusWidth=0):

        self.size = 24
        self.nSamples = nSamples
        self.nProfiles = nProfiles
        self.nChannels = nChannels
        self.adcResolution = adcResolution
        self.pciDioBusWidth = pciDioBusWidth

    def read(self, fp):
        self.length = 0
        try:
            startFp = fp.tell()
        except Exception as e:
            startFp = None
            pass

        try:
            if hasattr(fp, 'read'):
                header = numpy.fromfile(fp, SYSTEM_STRUCTURE, 1)
            else:
                header = numpy.fromstring(fp, SYSTEM_STRUCTURE, 1)
        except Exception as e:
            print("System Header: " + str(e))
            return 0

        self.size = header['nSize'][0]
        self.nSamples = header['nNumSamples'][0]
        self.nProfiles = header['nNumProfiles'][0]
        self.nChannels = header['nNumChannels'][0]
        self.adcResolution = header['nADCResolution'][0]
        self.pciDioBusWidth = header['nPCDIOBusWidth'][0]

        if startFp is not None:
            endFp = self.size + startFp

            if fp.tell() > endFp:
                sys.stderr.write(
                    "Warning %s: Size value read from System Header is lower than it has to be\n" % fp.name)
                return 0

            if fp.tell() < endFp:
                sys.stderr.write(
                    "Warning %s: Size value read from System Header size is greater than it has to be\n" % fp.name)
                return 0

        self.length = header.nbytes
        return 1

    def write(self, fp):

        headerTuple = (self.size, self.nSamples, self.nProfiles,
                       self.nChannels, self.adcResolution, self.pciDioBusWidth)
        header = numpy.array(headerTuple, SYSTEM_STRUCTURE)
        header.tofile(fp)

        return 1


class RadarControllerHeader(Header):

    expType = None
    nTx = None
    ipp = None
    txA = None
    txB = None
    nWindows = None
    numTaus = None
    codeType = None
    line6Function = None
    line5Function = None
    fClock = None
    prePulseBefore = None
    prePulseAfter = None
    rangeIpp = None
    rangeTxA = None
    rangeTxB = None
    structure = RADAR_STRUCTURE
    __size = None

    def __init__(self, expType=2, nTx=1,
                 ipp=None, txA=0, txB=0,
                 nWindows=None, nHeights=None, firstHeight=None, deltaHeight=None,
                 numTaus=0, line6Function=0, line5Function=0, fClock=None,
                 prePulseBefore=0, prePulseAfter=0,
                 codeType=0, nCode=0, nBaud=0, code=None,
                 flip1=0, flip2=0):

        #         self.size = 116
        self.expType = expType
        self.nTx = nTx
        self.ipp = ipp
        self.txA = txA
        self.txB = txB
        self.rangeIpp = ipp
        self.rangeTxA = txA
        self.rangeTxB = txB

        self.nWindows = nWindows
        self.numTaus = numTaus
        self.codeType = codeType
        self.line6Function = line6Function
        self.line5Function = line5Function
        self.fClock = fClock
        self.prePulseBefore = prePulseBefore
        self.prePulseAfter = prePulseAfter

        self.nHeights = nHeights
        self.firstHeight = firstHeight
        self.deltaHeight = deltaHeight
        self.samplesWin = nHeights

        self.nCode = nCode
        self.nBaud = nBaud
        self.code = code
        self.flip1 = flip1
        self.flip2 = flip2

        self.code_size = int(numpy.ceil(self.nBaud / 32.)) * self.nCode * 4
#         self.dynamic = numpy.array([],numpy.dtype('byte'))

        if self.fClock is None and self.deltaHeight is not None:
            self.fClock = 0.15 / (deltaHeight * 1e-6)  # 0.15Km / (height * 1u)

    def read(self, fp):
        self.length = 0
        try:
            startFp = fp.tell()
        except Exception as e:
            startFp = None
            pass

        try:
            if hasattr(fp, 'read'):
                header = numpy.fromfile(fp, RADAR_STRUCTURE, 1)
            else:
                header = numpy.fromstring(fp, RADAR_STRUCTURE, 1)
            self.length += header.nbytes
        except Exception as e:
            print("RadarControllerHeader: " + str(e))
            return 0

        size = int(header['nSize'][0])
        self.expType = int(header['nExpType'][0])
        self.nTx = int(header['nNTx'][0])
        self.ipp = float(header['fIpp'][0])
        self.txA = float(header['fTxA'][0])
        self.txB = float(header['fTxB'][0])
        self.nWindows = int(header['nNumWindows'][0])
        self.numTaus = int(header['nNumTaus'][0])
        self.codeType = int(header['nCodeType'][0])
        self.line6Function = int(header['nLine6Function'][0])
        self.line5Function = int(header['nLine5Function'][0])
        self.fClock = float(header['fClock'][0])
        self.prePulseBefore = int(header['nPrePulseBefore'][0])
        self.prePulseAfter = int(header['nPrePulseAfter'][0])
        self.rangeIpp = header['sRangeIPP'][0]
        self.rangeTxA = header['sRangeTxA'][0]
        self.rangeTxB = header['sRangeTxB'][0]

        try:
            if hasattr(fp, 'read'):
                samplingWindow = numpy.fromfile(
                    fp, SAMPLING_STRUCTURE, self.nWindows)
            else:
                samplingWindow = numpy.fromstring(
                    fp[self.length:], SAMPLING_STRUCTURE, self.nWindows)
            self.length += samplingWindow.nbytes
        except Exception as e:
            print("RadarControllerHeader: " + str(e))
            return 0
        self.nHeights = int(numpy.sum(samplingWindow['nsa']))
        self.firstHeight = samplingWindow['h0']
        self.deltaHeight = samplingWindow['dh']
        self.samplesWin = samplingWindow['nsa']

        try:
            if hasattr(fp, 'read'):
                self.Taus = numpy.fromfile(fp, '<f4', self.numTaus)
            else:
                self.Taus = numpy.fromstring(
                    fp[self.length:], '<f4', self.numTaus)
            self.length += self.Taus.nbytes
        except Exception as e:
            print("RadarControllerHeader: " + str(e))
            return 0

        self.code_size = 0
        if self.codeType != 0:

            try:
                if hasattr(fp, 'read'):
                    self.nCode = numpy.fromfile(fp, '<u4', 1)[0]
                    self.length += self.nCode.nbytes
                    self.nBaud = numpy.fromfile(fp, '<u4', 1)[0]
                    self.length += self.nBaud.nbytes
                else:
                    self.nCode = numpy.fromstring(
                        fp[self.length:], '<u4', 1)[0]
                    self.length += self.nCode.nbytes
                    self.nBaud = numpy.fromstring(
                        fp[self.length:], '<u4', 1)[0]
                    self.length += self.nBaud.nbytes
            except Exception as e:
                print("RadarControllerHeader: " + str(e))
                return 0
            code = numpy.empty([self.nCode, self.nBaud], dtype='i1')

            for ic in range(self.nCode):
                try:
                    if hasattr(fp, 'read'):
                        temp = numpy.fromfile(fp, 'u4', int(
                            numpy.ceil(self.nBaud / 32.)))
                    else:
                        temp = numpy.fromstring(
                            fp, 'u4', int(numpy.ceil(self.nBaud / 32.)))
                    self.length += temp.nbytes
                except Exception as e:
                    print("RadarControllerHeader: " + str(e))
                    return 0

                for ib in range(self.nBaud - 1, -1, -1):
                    code[ic, ib] = temp[int(ib / 32)] % 2
                    temp[int(ib / 32)] = temp[int(ib / 32)] / 2

            self.code = 2.0 * code - 1.0
            self.code_size = int(numpy.ceil(self.nBaud / 32.)) * self.nCode * 4

#             if self.line5Function == RCfunction.FLIP:
#                 self.flip1 = numpy.fromfile(fp,'<u4',1)
#
#             if self.line6Function == RCfunction.FLIP:
#                 self.flip2 = numpy.fromfile(fp,'<u4',1)
        if startFp is not None:
            endFp = size + startFp

            if fp.tell() != endFp:
                #             fp.seek(endFp)
                print("%s: Radar Controller Header size is not consistent: from data [%d] != from header field [%d]" % (fp.name, fp.tell() - startFp, size))
    #             return 0

            if fp.tell() > endFp:
                sys.stderr.write(
                    "Warning %s: Size value read from Radar Controller header is lower than it has to be\n" % fp.name)
    #             return 0

            if fp.tell() < endFp:
                sys.stderr.write(
                    "Warning %s: Size value read from Radar Controller header is greater than it has to be\n" % fp.name)

        return 1

    def write(self, fp):

        headerTuple = (self.size,
                       self.expType,
                       self.nTx,
                       self.ipp,
                       self.txA,
                       self.txB,
                       self.nWindows,
                       self.numTaus,
                       self.codeType,
                       self.line6Function,
                       self.line5Function,
                       self.fClock,
                       self.prePulseBefore,
                       self.prePulseAfter,
                       self.rangeIpp,
                       self.rangeTxA,
                       self.rangeTxB)

        header = numpy.array(headerTuple, RADAR_STRUCTURE)
        header.tofile(fp)

        sampleWindowTuple = (
            self.firstHeight, self.deltaHeight, self.samplesWin)
        samplingWindow = numpy.array(sampleWindowTuple, SAMPLING_STRUCTURE)
        samplingWindow.tofile(fp)

        if self.numTaus > 0:
            self.Taus.tofile(fp)

        if self.codeType != 0:
            nCode = numpy.array(self.nCode, '<u4')
            nCode.tofile(fp)
            nBaud = numpy.array(self.nBaud, '<u4')
            nBaud.tofile(fp)
            code1 = (self.code + 1.0) / 2.

            for ic in range(self.nCode):
                tempx = numpy.zeros(int(numpy.ceil(self.nBaud / 32.)))
                start = 0
                end = 32
                for i in range(len(tempx)):
                    code_selected = code1[ic, start:end]
                    for j in range(len(code_selected) - 1, -1, -1):
                        if code_selected[j] == 1:
                            tempx[i] = tempx[i] + \
                                2**(len(code_selected) - 1 - j)
                    start = start + 32
                    end = end + 32

                tempx = tempx.astype('u4')
                tempx.tofile(fp)

#         if self.line5Function == RCfunction.FLIP:
#             self.flip1.tofile(fp)
#
#         if self.line6Function == RCfunction.FLIP:
#             self.flip2.tofile(fp)

        return 1

    def get_ippSeconds(self):
        '''
        '''
        ippSeconds = 2.0 * 1000 * self.ipp / SPEED_OF_LIGHT

        return ippSeconds

    def set_ippSeconds(self, ippSeconds):
        '''
        '''

        self.ipp = ippSeconds * SPEED_OF_LIGHT / (2.0 * 1000)

        return

    def get_size(self):

        self.__size = 116 + 12 * self.nWindows + 4 * self.numTaus

        if self.codeType != 0:
            self.__size += 4 + 4 + 4 * self.nCode * \
                numpy.ceil(self.nBaud / 32.)

        return self.__size

    def set_size(self, value):

        raise IOError("size is a property and it cannot be set, just read")

        return

    ippSeconds = property(get_ippSeconds, set_ippSeconds)
    size = property(get_size, set_size)


class ProcessingHeader(Header):

    #     size = None
    dtype = None
    blockSize = None
    profilesPerBlock = None
    dataBlocksPerFile = None
    nWindows = None
    processFlags = None
    nCohInt = None
    nIncohInt = None
    totalSpectra = None
    structure = PROCESSING_STRUCTURE
    flag_dc = None
    flag_cspc = None

    def __init__(self, dtype=0, blockSize=0, profilesPerBlock=0, dataBlocksPerFile=0, nWindows=0, processFlags=0, nCohInt=0,
                 nIncohInt=0, totalSpectra=0, nHeights=0, firstHeight=0, deltaHeight=0, samplesWin=0, spectraComb=0, nCode=0,
                 code=0, nBaud=None, shif_fft=False, flag_dc=False, flag_cspc=False, flag_decode=False, flag_deflip=False
                 ):

        #         self.size = 0
        self.dtype = dtype
        self.blockSize = blockSize
        self.profilesPerBlock = 0
        self.dataBlocksPerFile = 0
        self.nWindows = 0
        self.processFlags = 0
        self.nCohInt = 0
        self.nIncohInt = 0
        self.totalSpectra = 0

        self.nHeights = 0
        self.firstHeight = 0
        self.deltaHeight = 0
        self.samplesWin = 0
        self.spectraComb = 0
        self.nCode = None
        self.code = None
        self.nBaud = None

        self.shif_fft = False
        self.flag_dc = False
        self.flag_cspc = False
        self.flag_decode = False
        self.flag_deflip = False
        self.length = 0

    def read(self, fp):
        self.length = 0
        try:
            startFp = fp.tell()
        except Exception as e:
            startFp = None
            pass

        try:
            if hasattr(fp, 'read'):
                header = numpy.fromfile(fp, PROCESSING_STRUCTURE, 1)
            else:
                header = numpy.fromstring(fp, PROCESSING_STRUCTURE, 1)
            self.length += header.nbytes
        except Exception as e:
            print("ProcessingHeader: " + str(e))
            return 0

        size = int(header['nSize'][0])
        self.dtype = int(header['nDataType'][0])
        self.blockSize = int(header['nSizeOfDataBlock'][0])
        self.profilesPerBlock = int(header['nProfilesperBlock'][0])
        self.dataBlocksPerFile = int(header['nDataBlocksperFile'][0])
        self.nWindows = int(header['nNumWindows'][0])
        self.processFlags = header['nProcessFlags']
        self.nCohInt = int(header['nCoherentIntegrations'][0])
        self.nIncohInt = int(header['nIncoherentIntegrations'][0])
        self.totalSpectra = int(header['nTotalSpectra'][0])

        try:
            if hasattr(fp, 'read'):
                samplingWindow = numpy.fromfile(
                    fp, SAMPLING_STRUCTURE, self.nWindows)
            else:
                samplingWindow = numpy.fromstring(
                    fp[self.length:], SAMPLING_STRUCTURE, self.nWindows)
            self.length += samplingWindow.nbytes
        except Exception as e:
            print("ProcessingHeader: " + str(e))
            return 0

        self.nHeights = int(numpy.sum(samplingWindow['nsa']))
        self.firstHeight = float(samplingWindow['h0'][0])
        self.deltaHeight = float(samplingWindow['dh'][0])
        self.samplesWin = samplingWindow['nsa'][0]

        try:
            if hasattr(fp, 'read'):
                self.spectraComb = numpy.fromfile(
                    fp, 'u1', 2 * self.totalSpectra)
            else:
                self.spectraComb = numpy.fromstring(
                    fp[self.length:], 'u1', 2 * self.totalSpectra)
            self.length += self.spectraComb.nbytes
        except Exception as e:
            print("ProcessingHeader: " + str(e))
            return 0

        if ((self.processFlags & PROCFLAG.DEFINE_PROCESS_CODE) == PROCFLAG.DEFINE_PROCESS_CODE):
            self.nCode = int(numpy.fromfile(fp, '<u4', 1))
            self.nBaud = int(numpy.fromfile(fp, '<u4', 1))
            self.code = numpy.fromfile(
                fp, '<f4', self.nCode * self.nBaud).reshape(self.nCode, self.nBaud)

        if ((self.processFlags & PROCFLAG.EXP_NAME_ESP) == PROCFLAG.EXP_NAME_ESP):
            exp_name_len = int(numpy.fromfile(fp, '<u4', 1))
            exp_name = numpy.fromfile(fp, 'u1', exp_name_len + 1)

        if ((self.processFlags & PROCFLAG.SHIFT_FFT_DATA) == PROCFLAG.SHIFT_FFT_DATA):
            self.shif_fft = True
        else:
            self.shif_fft = False

        if ((self.processFlags & PROCFLAG.SAVE_CHANNELS_DC) == PROCFLAG.SAVE_CHANNELS_DC):
            self.flag_dc = True
        else:
            self.flag_dc = False

        if ((self.processFlags & PROCFLAG.DECODE_DATA) == PROCFLAG.DECODE_DATA):
            self.flag_decode = True
        else:
            self.flag_decode = False

        if ((self.processFlags & PROCFLAG.DEFLIP_DATA) == PROCFLAG.DEFLIP_DATA):
            self.flag_deflip = True
        else:
            self.flag_deflip = False

        nChannels = 0
        nPairs = 0
        pairList = []

        for i in range(0, self.totalSpectra * 2, 2):
            if self.spectraComb[i] == self.spectraComb[i + 1]:
                nChannels = nChannels + 1  # par de canales iguales
            else:
                nPairs = nPairs + 1  # par de canales diferentes
                pairList.append((self.spectraComb[i], self.spectraComb[i + 1]))

        self.flag_cspc = False
        if nPairs > 0:
            self.flag_cspc = True

        if startFp is not None:
            endFp = size + startFp
            if fp.tell() > endFp:
                sys.stderr.write(
                    "Warning: Processing header size is lower than it has to be")
                return 0

            if fp.tell() < endFp:
                sys.stderr.write(
                    "Warning: Processing header size is greater than it is considered")

        return 1

    def write(self, fp):
        # Clear DEFINE_PROCESS_CODE
        self.processFlags = self.processFlags & (~PROCFLAG.DEFINE_PROCESS_CODE)

        headerTuple = (self.size,
                       self.dtype,
                       self.blockSize,
                       self.profilesPerBlock,
                       self.dataBlocksPerFile,
                       self.nWindows,
                       self.processFlags,
                       self.nCohInt,
                       self.nIncohInt,
                       self.totalSpectra)

        header = numpy.array(headerTuple, PROCESSING_STRUCTURE)
        header.tofile(fp)

        if self.nWindows != 0:
            sampleWindowTuple = (
                self.firstHeight, self.deltaHeight, self.samplesWin)
            samplingWindow = numpy.array(sampleWindowTuple, SAMPLING_STRUCTURE)
            samplingWindow.tofile(fp)

        if self.totalSpectra != 0:
            #             spectraComb = numpy.array([],numpy.dtype('u1'))
            spectraComb = self.spectraComb
            spectraComb.tofile(fp)

#         if self.processFlags & PROCFLAG.DEFINE_PROCESS_CODE == PROCFLAG.DEFINE_PROCESS_CODE:
#             nCode = numpy.array([self.nCode], numpy.dtype('u4')) #Probar con un dato que almacene codigo, hasta el momento no se hizo la prueba
#             nCode.tofile(fp)
#
#             nBaud = numpy.array([self.nBaud], numpy.dtype('u4'))
#             nBaud.tofile(fp)
#
#             code = self.code.reshape(self.nCode*self.nBaud)
#             code = code.astype(numpy.dtype('<f4'))
#             code.tofile(fp)

        return 1

    def get_size(self):

        self.__size = 40 + 12 * self.nWindows + 2 * self.totalSpectra

#         if self.processFlags & PROCFLAG.DEFINE_PROCESS_CODE == PROCFLAG.DEFINE_PROCESS_CODE:
#             self.__size += 4 + 4 + 4*self.nCode*numpy.ceil(self.nBaud/32.)
#             self.__size += 4 + 4 + 4 * self.nCode * self.nBaud

        return self.__size

    def set_size(self, value):

        raise IOError("size is a property and it cannot be set, just read")

        return

    size = property(get_size, set_size)


class RCfunction:
    NONE = 0
    FLIP = 1
    CODE = 2
    SAMPLING = 3
    LIN6DIV256 = 4
    SYNCHRO = 5


class nCodeType:
    NONE = 0
    USERDEFINE = 1
    BARKER2 = 2
    BARKER3 = 3
    BARKER4 = 4
    BARKER5 = 5
    BARKER7 = 6
    BARKER11 = 7
    BARKER13 = 8
    AC128 = 9
    COMPLEMENTARYCODE2 = 10
    COMPLEMENTARYCODE4 = 11
    COMPLEMENTARYCODE8 = 12
    COMPLEMENTARYCODE16 = 13
    COMPLEMENTARYCODE32 = 14
    COMPLEMENTARYCODE64 = 15
    COMPLEMENTARYCODE128 = 16
    CODE_BINARY28 = 17


class PROCFLAG:

    COHERENT_INTEGRATION = numpy.uint32(0x00000001)
    DECODE_DATA = numpy.uint32(0x00000002)
    SPECTRA_CALC = numpy.uint32(0x00000004)
    INCOHERENT_INTEGRATION = numpy.uint32(0x00000008)
    POST_COHERENT_INTEGRATION = numpy.uint32(0x00000010)
    SHIFT_FFT_DATA = numpy.uint32(0x00000020)

    DATATYPE_CHAR = numpy.uint32(0x00000040)
    DATATYPE_SHORT = numpy.uint32(0x00000080)
    DATATYPE_LONG = numpy.uint32(0x00000100)
    DATATYPE_INT64 = numpy.uint32(0x00000200)
    DATATYPE_FLOAT = numpy.uint32(0x00000400)
    DATATYPE_DOUBLE = numpy.uint32(0x00000800)

    DATAARRANGE_CONTIGUOUS_CH = numpy.uint32(0x00001000)
    DATAARRANGE_CONTIGUOUS_H = numpy.uint32(0x00002000)
    DATAARRANGE_CONTIGUOUS_P = numpy.uint32(0x00004000)

    SAVE_CHANNELS_DC = numpy.uint32(0x00008000)
    DEFLIP_DATA = numpy.uint32(0x00010000)
    DEFINE_PROCESS_CODE = numpy.uint32(0x00020000)

    ACQ_SYS_NATALIA = numpy.uint32(0x00040000)
    ACQ_SYS_ECHOTEK = numpy.uint32(0x00080000)
    ACQ_SYS_ADRXD = numpy.uint32(0x000C0000)
    ACQ_SYS_JULIA = numpy.uint32(0x00100000)
    ACQ_SYS_XXXXXX = numpy.uint32(0x00140000)

    EXP_NAME_ESP = numpy.uint32(0x00200000)
    CHANNEL_NAMES_ESP = numpy.uint32(0x00400000)

    OPERATION_MASK = numpy.uint32(0x0000003F)
    DATATYPE_MASK = numpy.uint32(0x00000FC0)
    DATAARRANGE_MASK = numpy.uint32(0x00007000)
    ACQ_SYS_MASK = numpy.uint32(0x001C0000)


dtype0 = numpy.dtype([('real', '<i1'), ('imag', '<i1')])
dtype1 = numpy.dtype([('real', '<i2'), ('imag', '<i2')])
dtype2 = numpy.dtype([('real', '<i4'), ('imag', '<i4')])
dtype3 = numpy.dtype([('real', '<i8'), ('imag', '<i8')])
dtype4 = numpy.dtype([('real', '<f4'), ('imag', '<f4')])
dtype5 = numpy.dtype([('real', '<f8'), ('imag', '<f8')])

NUMPY_DTYPE_LIST = [dtype0, dtype1, dtype2, dtype3, dtype4, dtype5]

PROCFLAG_DTYPE_LIST = [PROCFLAG.DATATYPE_CHAR,
                       PROCFLAG.DATATYPE_SHORT,
                       PROCFLAG.DATATYPE_LONG,
                       PROCFLAG.DATATYPE_INT64,
                       PROCFLAG.DATATYPE_FLOAT,
                       PROCFLAG.DATATYPE_DOUBLE]

DTYPE_WIDTH = [1, 2, 4, 8, 4, 8]


def get_dtype_index(numpy_dtype):

    index = None

    for i in range(len(NUMPY_DTYPE_LIST)):
        if numpy_dtype == NUMPY_DTYPE_LIST[i]:
            index = i
            break

    return index


def get_numpy_dtype(index):

    return NUMPY_DTYPE_LIST[index]


def get_procflag_dtype(index):

    return PROCFLAG_DTYPE_LIST[index]


def get_dtype_width(index):

    return DTYPE_WIDTH[index]


# Copyright (c) 2012-2020 Jicamarca Radio Observatory
# All rights reserved.
#
# Distributed under the terms of the BSD 3-clause license.
"""Utilities for publish/send data, files & plots over different protocols
"""

import os
import glob
import time
import json
import numpy
import zmq
import datetime
import ftplib
from functools import wraps
from threading import Thread
from multiprocessing import Process

from schainpy.model.proc.jroproc_base import Operation, ProcessingUnit, MPDecorator
from schainpy.model.data.jrodata import JROData
from schainpy.utils import log


PLOT_CODES = {
    'rti': 0,            # Range time intensity (RTI).
    'spc': 1,            # Spectra (and Cross-spectra) information.
    'cspc': 2,           # Cross-Correlation information.
    'coh': 3,            # Coherence map.
    'base': 4,           # Base lines graphic.
    'row': 5,            # Row Spectra.
    'total': 6,          # Total Power.
    'drift': 7,          # Drifts graphics.
    'height': 8,         # Height profile.
    'phase': 9,          # Signal Phase.
    'power': 16,
    'noise': 17,
    'beacon': 18,
    'wind': 22,
    'skymap': 23,
    'Unknown': 24,
    'V-E': 25,          # PIP Velocity.
    'Z-E': 26,          # PIP Reflectivity.
    'V-A': 27,          # RHI Velocity.
    'Z-A': 28,          # RHI Reflectivity.
}

def get_plot_code(s):
    label = s.split('_')[0]
    codes = [key for key in PLOT_CODES if key in label]
    if codes:        
        return PLOT_CODES[codes[0]]
    else:
        return 24


class PublishData(Operation):
    '''
    Operation to send data over zmq.
    '''

    __attrs__ = ['host', 'port', 'delay', 'verbose']

    def setup(self, server='zmq.pipe', delay=0, verbose=True, **kwargs):
        self.counter = 0
        self.delay = kwargs.get('delay', 0)
        self.cnt = 0
        self.verbose = verbose        
        context = zmq.Context()
        self.zmq_socket = context.socket(zmq.PUSH)
        server = kwargs.get('server', 'zmq.pipe')

        if 'tcp://' in server:
            address = server
        else:
            address = 'ipc:///tmp/%s' % server

        self.zmq_socket.connect(address)
        time.sleep(1)


    def publish_data(self):
        self.dataOut.finished = False
        
        if self.verbose:
            log.log(
                'Sending {} - {}'.format(self.dataOut.type, self.dataOut.datatime),
                self.name
            )
        self.zmq_socket.send_pyobj(self.dataOut)

    def run(self, dataOut, **kwargs):
        self.dataOut = dataOut
        if not self.isConfig:
            self.setup(**kwargs)
            self.isConfig = True

        self.publish_data()
        time.sleep(self.delay)

    def close(self):
        
        self.dataOut.finished = True
        self.zmq_socket.send_pyobj(self.dataOut)
        time.sleep(0.1)
        self.zmq_socket.close()
        

class ReceiverData(ProcessingUnit):

    __attrs__ = ['server']

    def __init__(self, **kwargs):

        ProcessingUnit.__init__(self, **kwargs)

        self.isConfig = False
        server = kwargs.get('server', 'zmq.pipe')
        if 'tcp://' in server:
            address = server
        else:
            address = 'ipc:///tmp/%s' % server

        self.address = address
        self.dataOut = JROData()

    def setup(self):

        self.context = zmq.Context()
        self.receiver = self.context.socket(zmq.PULL)
        self.receiver.bind(self.address)
        time.sleep(0.5)
        log.success('ReceiverData from {}'.format(self.address))


    def run(self):

        if not self.isConfig:
            self.setup()
            self.isConfig = True

        self.dataOut = self.receiver.recv_pyobj()
        log.log('{} - {}'.format(self.dataOut.type,
                                 self.dataOut.datatime.ctime(),),
                'Receiving')

@MPDecorator
class SendToFTP(Operation):
    """Operation for send files over FTP

    This operation is used to send files over FTP, you can send different files
    from different folders by adding as many `pattern` as you wish.

    Parameters:
    -----------
    server : str
        FTP server address.
    username : str
        FTP username
    password : str
        FTP password
    timeout : int
        timeout to restart the connection
    patternX : list
        detail of files to be send must have the following order: local, remote
        ext, period, exp_code, sub_exp_code 
    
    Example:
    --------
    
    ftp = proc_unit.addOperation(name='SendToFTP', optype='external')
    ftp.addParameter(name='server', value='jro-app.igp.gob.pe')
    ftp.addParameter(name='username', value='wmaster')
    ftp.addParameter(name='password', value='mst2010vhf')
    ftp.addParameter(
        name='pattern1', 
        value='/local/path/rti,/remote/path,png,300,11,0'
        )
    ftp.addParameter(
        name='pattern2',
        value='/local/path/spc,/remote/path,png,300,11,0'
        )
    ftp.addParameter(
        name='pattern3', 
        value='/local/path/param,/remote/path,hdf5,300,,'
        )

    """
    
    __attrs__ = ['server', 'username', 'password', 'timeout', 'patternX']

    def __init__(self):
        '''
        '''
        Operation.__init__(self)
        self.ftp = None
        self.ready = False

    def setup(self, server, username, password, timeout, **kwargs):
        '''
        '''

        self.server = server
        self.username = username
        self.password = password
        self.timeout = timeout
        self.patterns = []
        self.times = []
        self.latest = []
        for arg, value in kwargs.items():
            if 'pattern' in arg:
                self.patterns.append(value)
                self.times.append(0)
                self.latest.append('')

    def connect(self):
        '''
        '''

        log.log('Connecting to ftp://{}'.format(self.server), self.name)
        try:
            self.ftp = ftplib.FTP(self.server, timeout=self.timeout)
        except ftplib.all_errors:
            log.error('Server connection fail: {}'.format(self.server), self.name)
            if self.ftp is not None:
                self.ftp.close()
            self.ftp = None
            self.ready = False
            return 

        try:
            self.ftp.login(self.username, self.password)
        except ftplib.all_errors:
            log.error('The given username y/o password are incorrect', self.name)
            if self.ftp is not None:
                self.ftp.close()
            self.ftp = None
            self.ready = False
            return

        log.success('Connection success', self.name)
        self.ready = True
        return

    def check(self):

        try:
            self.ftp.voidcmd("NOOP")
        except:
            log.warning('Connection lost... trying to reconnect', self.name)
            if self.ftp is not None:
                self.ftp.close()
            self.ftp = None
            self.connect()

    def find_files(self, path, ext):

        files = glob.glob1(path.strip(), '*{}'.format(ext.strip()))
        files.sort()
        if files:
            return files[-1]
        return None

    def getftpname(self, filename, exp_code, sub_exp_code):

        thisDatetime = datetime.datetime.strptime(filename.split('_')[1], '%Y%m%d')
        YEAR_STR = '%4.4d' % thisDatetime.timetuple().tm_year
        DOY_STR = '%3.3d' % thisDatetime.timetuple().tm_yday
        exp_code = '%3.3d' % exp_code
        sub_exp_code = '%2.2d' % sub_exp_code
        plot_code = '%2.2d' % get_plot_code(filename)
        name = YEAR_STR + DOY_STR + '00' + exp_code + sub_exp_code + plot_code + '00.png'
        return name

    def upload(self, src, dst):

        log.log('Uploading {} -> {} '.format(
            src.split('/')[-1], dst.split('/')[-1]), 
            self.name, 
            nl=False
            )

        fp = open(src, 'rb')
        command = 'STOR {}'.format(dst)

        try:
            self.ftp.storbinary(command, fp, blocksize=1024)
        except Exception as e:
            log.error('{}'.format(e), self.name)
            return 0

        try:
            self.ftp.sendcmd('SITE CHMOD 755 {}'.format(dst))
        except Exception as e:
            log.error('{}'.format(e), self.name)
            return 0

        fp.close()
        log.success('OK', tag='')
        return 1
    
    def send_files(self):

        for x, pattern in enumerate(self.patterns):
            local, remote, ext, period, exp_code, sub_exp_code = pattern
            
            if (self.dataOut.utctime - self.times[x]) < int(period):
                continue

            srcname = self.find_files(local, ext)
            
            if srcname is None:
                continue
            
            if srcname == self.latest[x]:
                log.warning('File alreday uploaded {}'.format(srcname))                  
                continue
            
            if exp_code.strip():
                dstname = self.getftpname(srcname, int(exp_code), int(sub_exp_code))
            else:
                dstname = srcname                
            
            src = os.path.join(local, srcname)
            dst = os.path.join(remote.strip(), dstname)

            if self.upload(src, dst):
                self.times[x] = self.dataOut.utctime
                self.latest[x] = srcname            

    def run(self, dataOut, server, username, password, timeout=10, **kwargs):

        if not self.isConfig:
            self.setup(
                server=server, 
                username=username, 
                password=password, 
                timeout=timeout, 
                **kwargs
                )
            self.isConfig = True
            self.connect()
        
        self.dataOut = dataOut
        self.check()
        self.send_files()

    def close(self):

        if self.ftp is not None:
            self.ftp.close()



'''
SCHAINPY - LOG
    Simple helper for log standarization
    Usage:
        from schainpy.utils import log
        log.error('A kitten died beacuse of you')
        log.warning('You are doing it wrong but what the heck, I'll allow it)
        log.succes('YOU ROCK!')
    To create your own logger inside your class do it like this:
        from schainpy.utils import log
        awesomeLogger = log.makelogger("never gonna", bg="red", fg="white")
        awesomeLogger('give you up')
    which will look like this:
        [NEVER GONNA] - give you up
    with color red as background and white as foreground.
'''

import click


def warning(message, tag='Warning', nl=True):
    if tag:
        click.echo(click.style('[{}] {}'.format(tag, message), fg='yellow'), nl=nl)
    else:
        click.echo(click.style('{}'.format(message), fg='yellow'), nl=nl)
    pass


def error(message, tag='Error', nl=True):
    if tag:
        click.echo(click.style('[{}] {}'.format(tag, message), fg='red'), nl=nl)
    else:
        click.echo(click.style('{}'.format(message), fg='red'), nl=nl)
    pass


def success(message, tag='Success', nl=True):
    if tag:
        click.echo(click.style('[{}] {}'.format(tag, message), fg='green'), nl=nl)
    else:
        click.echo(click.style('{}'.format(message), fg='green'), nl=nl)
    pass


def log(message, tag='Info', nl=True):
    if tag:
        click.echo('[{}] {}'.format(tag, message), nl=nl)
    else:
        click.echo('{}'.format(message), nl=nl)
    pass


def makelogger(tag, bg='reset', fg='reset'):
    def func(message):
        click.echo(click.style('[{}] {}'.format(
            tag.upper(), message), bg=bg, fg=fg))
    return func
